import os

from dagster import asset, build_init_resource_context, materialize
from dagster._core.test_utils import environ

from dagster_aws.ssm import ParameterStoreResource, ParameterStoreTag, parameter_store_resource


def test_parameter_store_resource_structured_tags(mock_ssm_client) -> None:
    # Test using the structured input to ParameterStoreResource using the ParameterStoreTag input
    mock_ssm_client.put_parameter(
        Name="foo_param1",
        Value="foo_value1",
        Type="String",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value1"}],
    )
    mock_ssm_client.put_parameter(
        Name="foo_param2",
        Value="foo_value2",
        Type="String",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value2"}],
    )
    mock_ssm_client.put_parameter(
        Name="bar_param",
        Value="bar_value",
        Tags=[{"Key": "bar_tag_key", "Value": "bar_tag_value"}],
        Type="String",
    )
    mock_ssm_client.put_parameter(
        Name="path/based/param1",
        Value="path_param1",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value3"}],
        Type="String",
    )
    mock_ssm_client.put_parameter(
        Name="path/based/param1/nested",
        Value="path_param2",
        Type="String",
    )

    @asset
    def my_parameter_store_asset(parameter_store: ParameterStoreResource):
        assert parameter_store.fetch_parameters() == {
            "foo_param1": "foo_value1",
            "foo_param2": "foo_value2",
        }

    result = materialize(
        assets=[my_parameter_store_asset],
        resources={
            "parameter_store": ParameterStoreResource(
                parameter_tags=[
                    ParameterStoreTag(
                        key="foo_tag_key", values=["foo_tag_value1", "foo_tag_value2"]
                    )
                ]
            ),
        },
    )
    assert result.success


def test_parameter_store_resource(mock_ssm_client):
    mock_ssm_client.put_parameter(
        Name="foo_param1",
        Value="foo_value1",
        Type="String",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value1"}],
    )
    mock_ssm_client.put_parameter(
        Name="foo_param2",
        Value="foo_value2",
        Type="String",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value2"}],
    )
    mock_ssm_client.put_parameter(
        Name="bar_param",
        Value="bar_value",
        Tags=[{"Key": "bar_tag_key", "Value": "bar_tag_value"}],
        Type="String",
    )
    mock_ssm_client.put_parameter(
        Name="path/based/param1",
        Value="path_param1",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value3"}],
        Type="String",
    )
    mock_ssm_client.put_parameter(
        Name="path/based/param1/nested",
        Value="path_param2",
        Type="String",
    )
    mock_ssm_client.put_parameter(Name="other/path/param3", Value="param_param3", Type="String")
    with parameter_store_resource(
        build_init_resource_context(config={"parameters": ["foo_param1", "foo_param2"]})
    ) as param_map:
        assert param_map == {"foo_param1": "foo_value1", "foo_param2": "foo_value2"}

    with parameter_store_resource(
        build_init_resource_context(config={"parameter_paths": ["path/based", "other/path"]})
    ) as param_map:
        assert param_map == {
            "path/based/param1": "path_param1",
            "path/based/param1/nested": "path_param2",
            "other/path/param3": "param_param3",
        }

    with parameter_store_resource(
        build_init_resource_context(
            config={
                "parameter_tags": [
                    {"key": "foo_tag_key", "values": ["foo_tag_value1", "foo_tag_value2"]}
                ]
            }
        )
    ) as param_map:
        assert param_map == {"foo_param1": "foo_value1", "foo_param2": "foo_value2"}

    with parameter_store_resource(
        build_init_resource_context(
            config={
                "parameters": ["foo_param1"],
                "parameter_paths": ["path/based"],
                "parameter_tags": [{"key": "foo_tag_key", "values": ["foo_tag_value2"]}],
            }
        )
    ) as param_map:
        assert param_map == {
            "foo_param1": "foo_value1",
            "foo_param2": "foo_value2",
            "path/based/param1": "path_param1",
            "path/based/param1/nested": "path_param2",
        }


def test_parameters_added_to_environment(mock_ssm_client):
    mock_ssm_client.put_parameter(
        Name="foo_param1",
        Value="foo_value1",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value1"}],
        Type="String",
    )
    mock_ssm_client.put_parameter(
        Name="foo_param2",
        Value="foo_value2",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value2"}],
        Type="String",
    )

    assert os.getenv("foo_param1") is None
    assert os.getenv("foo_param2") is None

    with parameter_store_resource(
        build_init_resource_context(
            config={"parameters": ["foo_param1"], "add_to_environment": True}
        )
    ) as param_map:
        assert param_map == {"foo_param1": "foo_value1"}
        assert os.getenv("foo_param1") == "foo_value1"

    with environ({"foo_param2": "prior_foo2_value"}):
        assert os.getenv("foo_param2") == "prior_foo2_value"

        with parameter_store_resource(
            build_init_resource_context(
                config={"parameters": ["foo_param2"], "add_to_environment": True}
            )
        ) as param_map:
            assert param_map == {"foo_param2": "foo_value2"}
            assert os.getenv("foo_param2") == "foo_value2"


def test_parameter_store_resource_pythonic(mock_ssm_client) -> None:
    mock_ssm_client.put_parameter(
        Name="foo_param1",
        Value="foo_value1",
        Type="String",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value1"}],
    )
    mock_ssm_client.put_parameter(
        Name="foo_param2",
        Value="foo_value2",
        Type="String",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value2"}],
    )
    mock_ssm_client.put_parameter(
        Name="bar_param",
        Value="bar_value",
        Tags=[{"Key": "bar_tag_key", "Value": "bar_tag_value"}],
        Type="String",
    )
    mock_ssm_client.put_parameter(
        Name="path/based/param1",
        Value="path_param1",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value3"}],
        Type="String",
    )
    mock_ssm_client.put_parameter(
        Name="path/based/param1/nested",
        Value="path_param2",
        Type="String",
    )
    mock_ssm_client.put_parameter(Name="other/path/param3", Value="param_param3", Type="String")
    assert ParameterStoreResource(parameters=["foo_param1", "foo_param2"]).fetch_parameters() == {
        "foo_param1": "foo_value1",
        "foo_param2": "foo_value2",
    }

    assert ParameterStoreResource(
        parameter_paths=["path/based", "other/path"]
    ).fetch_parameters() == {
        "path/based/param1": "path_param1",
        "path/based/param1/nested": "path_param2",
        "other/path/param3": "param_param3",
    }

    assert ParameterStoreResource().fetch_parameters(
        parameter_paths=["path/based", "other/path"]
    ) == {
        "path/based/param1": "path_param1",
        "path/based/param1/nested": "path_param2",
        "other/path/param3": "param_param3",
    }

    assert ParameterStoreResource(
        parameter_tags=[
            ParameterStoreTag(key="foo_tag_key", values=["foo_tag_value1", "foo_tag_value2"])
        ]
    ).fetch_parameters() == {"foo_param1": "foo_value1", "foo_param2": "foo_value2"}

    assert ParameterStoreResource().fetch_parameters(
        parameter_tags=[
            ParameterStoreTag(key="foo_tag_key", values=["foo_tag_value1", "foo_tag_value2"])
        ]
    ) == {"foo_param1": "foo_value1", "foo_param2": "foo_value2"}

    assert ParameterStoreResource(
        parameters=["foo_param1"],
        parameter_paths=["path/based"],
        parameter_tags=[
            ParameterStoreTag(key="foo_tag_key", values=["foo_tag_value1", "foo_tag_value2"])
        ],
    ).fetch_parameters() == {
        "foo_param1": "foo_value1",
        "foo_param2": "foo_value2",
        "path/based/param1": "path_param1",
        "path/based/param1/nested": "path_param2",
    }

    assert ParameterStoreResource().fetch_parameters(
        parameters=["foo_param1"],
        parameter_paths=["path/based"],
        parameter_tags=[
            ParameterStoreTag(key="foo_tag_key", values=["foo_tag_value1", "foo_tag_value2"])
        ],
    ) == {
        "foo_param1": "foo_value1",
        "foo_param2": "foo_value2",
        "path/based/param1": "path_param1",
        "path/based/param1/nested": "path_param2",
    }


def test_parameters_added_to_environment_pythonic(mock_ssm_client) -> None:
    mock_ssm_client.put_parameter(
        Name="foo_param1",
        Value="foo_value1",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value1"}],
        Type="String",
    )
    mock_ssm_client.put_parameter(
        Name="foo_param2",
        Value="foo_value2",
        Tags=[{"Key": "foo_tag_key", "Value": "foo_tag_value2"}],
        Type="String",
    )

    assert os.getenv("foo_param1") is None
    assert os.getenv("foo_param2") is None

    with ParameterStoreResource(parameters=["foo_param1"]).parameters_in_environment() as param_map:
        assert param_map == {"foo_param1": "foo_value1"}
        assert os.getenv("foo_param1") == "foo_value1"

    with ParameterStoreResource().parameters_in_environment(parameters=["foo_param1"]) as param_map:
        assert param_map == {"foo_param1": "foo_value1"}
        assert os.getenv("foo_param1") == "foo_value1"

    with environ({"foo_param2": "prior_foo2_value"}):
        assert os.getenv("foo_param2") == "prior_foo2_value"

        with ParameterStoreResource(
            parameters=["foo_param2"]
        ).parameters_in_environment() as param_map:
            assert param_map == {"foo_param2": "foo_value2"}
            assert os.getenv("foo_param2") == "foo_value2"

        with ParameterStoreResource().parameters_in_environment(
            parameters=["foo_param2"]
        ) as param_map:
            assert param_map == {"foo_param2": "foo_value2"}
            assert os.getenv("foo_param2") == "foo_value2"



import pytest
from dagster._core.errors import DagsterInvalidConfigError
from dagster._core.test_utils import environ

from dagster_aws.ecs.container_context import EcsContainerContext


@pytest.fixture
def empty_container_context():
    return EcsContainerContext()


@pytest.fixture
def secrets_container_context(container_context_config):
    return EcsContainerContext.create_from_config(container_context_config)


@pytest.fixture
def other_secrets_container_context(other_container_context_config):
    return EcsContainerContext.create_from_config(other_container_context_config)


def test_empty_container_context(empty_container_context):
    assert empty_container_context.secrets == []
    assert empty_container_context.secrets_tags == []
    assert empty_container_context.env_vars == []


def test_invalid_config():
    with pytest.raises(
        DagsterInvalidConfigError, match="Errors while parsing ECS container context"
    ):
        EcsContainerContext.create_from_config(
            {"ecs": {"secrets": {"foo": "bar"}}}
        )  # invalid formatting


def test_merge(
    empty_container_context,
    secrets_container_context,
    other_secrets_container_context,
    configured_secret,
    other_configured_secret,
):
    assert secrets_container_context.secrets == [
        {"name": "HELLO", "valueFrom": configured_secret.arn + "/hello"},
    ]
    assert secrets_container_context.secrets_tags == ["dagster"]
    assert secrets_container_context.get_environment_dict() == {
        "FOO_ENV_VAR": "BAR_VALUE",
        "SHARED_KEY": "SHARED_VAL",
    }

    assert secrets_container_context.container_name == "foo"

    assert other_secrets_container_context.secrets == [
        {"name": "GOODBYE", "valueFrom": other_configured_secret.arn + "/goodbye"},
    ]

    assert other_secrets_container_context.secrets_tags == ["other_secret_tag"]

    assert other_secrets_container_context.container_name == "bar"

    with pytest.raises(
        Exception, match="Tried to load environment variable OTHER_FOO_ENV_VAR, but it was not set"
    ):
        other_secrets_container_context.get_environment_dict()

    with environ({"OTHER_FOO_ENV_VAR": "OTHER_BAR_VALUE"}):
        assert other_secrets_container_context.get_environment_dict() == {
            "OTHER_FOO_ENV_VAR": "OTHER_BAR_VALUE",
            "SHARED_OTHER_KEY": "SHARED_OTHER_VAL",
        }

    merged = secrets_container_context.merge(other_secrets_container_context)

    assert merged.secrets == [
        {"name": "GOODBYE", "valueFrom": other_configured_secret.arn + "/goodbye"},
        {"name": "HELLO", "valueFrom": configured_secret.arn + "/hello"},
    ]

    assert merged.secrets_tags == ["other_secret_tag", "dagster"]

    assert merged.container_name == "bar"

    assert merged.run_resources == {"cpu": "256", "memory": "8192", "ephemeral_storage": 100}
    assert merged.server_resources == {
        "cpu": "2048",
        "memory": "4096",
        "ephemeral_storage": 25,
        "replica_count": 2,
    }

    assert merged.server_ecs_tags == [
        {"key": "BAZ", "value": "QUUX"},
        {
            "key": "FOO",  # no value
        },
    ]
    assert merged.run_ecs_tags == [{"key": "GHI"}, {"key": "ABC", "value": "DEF"}]

    assert merged.task_role_arn == "other-task-role"
    assert merged.execution_role_arn == "other-fake-execution-role"

    assert merged.mount_points == [
        {
            "sourceVolume": "myOtherEfsVolume",
            "containerPath": "/mount/other/efs",
            "readOnly": True,
        },
        {
            "sourceVolume": "myEfsVolume",
            "containerPath": "/mount/efs",
            "readOnly": True,
        },
    ]

    assert merged.repository_credentials == "fake-secret-arn"

    assert merged.volumes == [
        {
            "name": "myOtherEfsVolume",
            "efsVolumeConfiguration": {
                "fileSystemId": "fs-5678",
                "rootDirectory": "/path/to/my/other/data",
            },
        },
        {
            "name": "myEfsVolume",
            "efsVolumeConfiguration": {
                "fileSystemId": "fs-1234",
                "rootDirectory": "/path/to/my/data",
            },
        },
    ]

    assert merged.server_sidecar_containers == [
        {
            "name": "OtherServerAgent",
            "image": "public.ecr.aws/other/agent:latest",
        },
        {
            "name": "DatadogAgent",
            "image": "public.ecr.aws/datadog/agent:latest",
            "environment": [
                {"name": "ECS_FARGATE", "value": "true"},
            ],
        },
    ]
    assert merged.server_health_check == other_secrets_container_context.server_health_check
    assert merged.run_sidecar_containers == [
        {
            "name": "OtherRunAgent",
            "image": "otherrun:latest",
        },
        {
            "name": "busyrun",
            "image": "busybox:latest",
        },
    ]

    with pytest.raises(
        Exception, match="Tried to load environment variable OTHER_FOO_ENV_VAR, but it was not set"
    ):
        merged.get_environment_dict()

    with environ({"OTHER_FOO_ENV_VAR": "OTHER_BAR_VALUE"}):
        assert merged.get_environment_dict() == {
            "FOO_ENV_VAR": "BAR_VALUE",
            "OTHER_FOO_ENV_VAR": "OTHER_BAR_VALUE",
            "SHARED_KEY": "SHARED_VAL",
            "SHARED_OTHER_KEY": "SHARED_OTHER_VAL",
        }

    assert (
        empty_container_context.merge(secrets_container_context).secrets
        == secrets_container_context.secrets
    )
    assert (
        empty_container_context.merge(secrets_container_context).secrets_tags
        == secrets_container_context.secrets_tags
    )



import os
import sys
import tempfile

import pytest
from botocore.exceptions import ClientError
from dagster import DagsterEventType, job, op
from dagster._core.instance import DagsterInstance, InstanceRef, InstanceType
from dagster._core.launcher import DefaultRunLauncher
from dagster._core.run_coordinator import DefaultRunCoordinator
from dagster._core.storage.compute_log_manager import ComputeIOType
from dagster._core.storage.event_log import SqliteEventLogStorage
from dagster._core.storage.local_compute_log_manager import IO_TYPE_EXTENSION
from dagster._core.storage.root import LocalArtifactStorage
from dagster._core.storage.runs import SqliteRunStorage
from dagster._core.test_utils import ensure_dagster_tests_import, environ, instance_for_test
from dagster._time import get_current_datetime

from dagster_aws.s3 import S3ComputeLogManager

ensure_dagster_tests_import()
from dagster_tests.storage_tests.test_compute_log_manager import TestComputeLogManager

HELLO_WORLD = "Hello World"
SEPARATOR = os.linesep if (os.name == "nt" and sys.version_info < (3,)) else "\n"
EXPECTED_LOGS = [
    'STEP_START - Started execution of step "easy".',
    'STEP_OUTPUT - Yielded output "result" of type "Any"',
    'STEP_SUCCESS - Finished execution of step "easy"',
]


def test_compute_log_manager(mock_s3_bucket):
    @op
    def easy(context):
        context.log.info("easy")
        print(HELLO_WORLD)  # noqa: T201
        return "easy"

    @job
    def simple():
        easy()

    with tempfile.TemporaryDirectory() as temp_dir:
        with environ({"DAGSTER_HOME": temp_dir}):
            run_store = SqliteRunStorage.from_local(temp_dir)
            event_store = SqliteEventLogStorage(temp_dir)
            manager = S3ComputeLogManager(
                bucket=mock_s3_bucket.name, prefix="my_prefix", local_dir=temp_dir
            )
            instance = DagsterInstance(
                instance_type=InstanceType.PERSISTENT,
                local_artifact_storage=LocalArtifactStorage(temp_dir),
                run_storage=run_store,
                event_storage=event_store,
                compute_log_manager=manager,
                run_coordinator=DefaultRunCoordinator(),
                run_launcher=DefaultRunLauncher(),
                ref=InstanceRef.from_dir(temp_dir),
                settings={"telemetry": {"enabled": False}},
            )
            result = simple.execute_in_process(instance=instance)
            capture_events = [
                event
                for event in result.all_events
                if event.event_type == DagsterEventType.LOGS_CAPTURED
            ]
            assert len(capture_events) == 1
            event = capture_events[0]
            file_key = event.logs_captured_data.file_key
            log_key = manager.build_log_key_for_run(result.run_id, file_key)

            # verify locally cached logs are deleted after they are captured
            local_path = manager._local_manager.get_captured_local_path(  # noqa: SLF001
                log_key, IO_TYPE_EXTENSION[ComputeIOType.STDOUT]
            )
            assert not os.path.exists(local_path)

            log_data = manager.get_log_data(log_key)
            stdout = log_data.stdout.decode("utf-8")  # pyright: ignore[reportOptionalMemberAccess]
            assert stdout == HELLO_WORLD + SEPARATOR

            stderr = log_data.stderr.decode("utf-8")  # pyright: ignore[reportOptionalMemberAccess]
            for expected in EXPECTED_LOGS:
                assert expected in stderr

            # Check S3 directly
            s3_object = mock_s3_bucket.Object(
                key=manager._s3_key(log_key, ComputeIOType.STDERR)  # noqa: SLF001
            )
            stderr_s3 = s3_object.get()["Body"].read().decode("utf-8")
            for expected in EXPECTED_LOGS:
                assert expected in stderr_s3

            log_data = manager.get_log_data(log_key)

            # Re-downloads the data to the local filesystem again
            assert os.path.exists(local_path)

            stdout = log_data.stdout.decode("utf-8")  # pyright: ignore[reportOptionalMemberAccess]
            assert stdout == HELLO_WORLD + SEPARATOR

            stderr = log_data.stderr.decode("utf-8")  # pyright: ignore[reportOptionalMemberAccess]
            for expected in EXPECTED_LOGS:
                assert expected in stderr


def test_compute_log_manager_from_config(mock_s3_bucket):
    s3_prefix = "foobar"

    dagster_yaml = f"""
compute_logs:
  module: dagster_aws.s3.compute_log_manager
  class: S3ComputeLogManager
  config:
    bucket: "{mock_s3_bucket.name}"
    local_dir: "/tmp/cool"
    prefix: "{s3_prefix}"
"""

    with tempfile.TemporaryDirectory() as tempdir:
        with open(os.path.join(tempdir, "dagster.yaml"), "wb") as f:
            f.write(dagster_yaml.encode("utf-8"))

        instance = DagsterInstance.from_config(tempdir)
    assert instance.compute_log_manager._s3_bucket == mock_s3_bucket.name  # noqa: SLF001  # pyright: ignore[reportAttributeAccessIssue]
    assert instance.compute_log_manager._s3_prefix == s3_prefix  # noqa: SLF001  # pyright: ignore[reportAttributeAccessIssue]


def test_compute_log_manager_skip_empty_upload(mock_s3_bucket):
    @op
    def easy(context):
        context.log.info("easy")

    @job
    def simple():
        easy()

    with tempfile.TemporaryDirectory() as temp_dir:
        with environ({"DAGSTER_HOME": temp_dir}):
            run_store = SqliteRunStorage.from_local(temp_dir)
            event_store = SqliteEventLogStorage(temp_dir)
            PREFIX = "my_prefix"
            manager = S3ComputeLogManager(
                bucket=mock_s3_bucket.name, prefix=PREFIX, skip_empty_files=True
            )
            instance = DagsterInstance(
                instance_type=InstanceType.PERSISTENT,
                local_artifact_storage=LocalArtifactStorage(temp_dir),
                run_storage=run_store,
                event_storage=event_store,
                compute_log_manager=manager,
                run_coordinator=DefaultRunCoordinator(),
                run_launcher=DefaultRunLauncher(),
                ref=InstanceRef.from_dir(temp_dir),
                settings={"telemetry": {"enabled": False}},
            )
            result = simple.execute_in_process(instance=instance)
            capture_events = [
                event
                for event in result.all_events
                if event.event_type == DagsterEventType.LOGS_CAPTURED
            ]
            assert len(capture_events) == 1
            event = capture_events[0]
            file_key = event.logs_captured_data.file_key
            log_key = manager.build_log_key_for_run(result.run_id, file_key)
            stderr_object = mock_s3_bucket.Object(
                key=manager._s3_key(log_key, ComputeIOType.STDERR)  # noqa: SLF001
            )
            assert stderr_object

            with pytest.raises(ClientError):
                # stdout is not uploaded because we do not print anything to stdout
                mock_s3_bucket.Object(
                    key=manager._s3_key(log_key, ComputeIOType.STDOUT)  # noqa: SLF001
                ).get()


def test_blank_compute_logs(mock_s3_bucket):
    with tempfile.TemporaryDirectory() as temp_dir:
        manager = S3ComputeLogManager(
            bucket=mock_s3_bucket.name, prefix="my_prefix", local_dir=temp_dir
        )

        # simulate subscription to an in-progress run, where there is no key in the bucket
        log_data = manager.get_log_data(["my_run_id", "compute_logs", "my_step_key"])

        assert not log_data.stdout
        assert not log_data.stderr


def test_prefix_filter(mock_s3_bucket):
    s3_prefix = "foo/bar/"  # note the trailing slash

    with tempfile.TemporaryDirectory() as temp_dir:
        manager = S3ComputeLogManager(
            bucket=mock_s3_bucket.name, prefix=s3_prefix, local_dir=temp_dir
        )
        log_key = ["arbitrary", "log", "key"]
        with manager.open_log_stream(log_key, ComputeIOType.STDERR) as write_stream:
            write_stream.write("hello hello")  # pyright: ignore[reportOptionalMemberAccess]

        s3_object = mock_s3_bucket.Object(key="foo/bar/storage/arbitrary/log/key.err")
        logs = s3_object.get()["Body"].read().decode("utf-8")
        assert logs == "hello hello"


def test_get_log_keys_for_log_key_prefix(mock_s3_bucket):
    evaluation_time = get_current_datetime()
    s3_prefix = "foo/bar/"  # note the trailing slash

    with tempfile.TemporaryDirectory() as temp_dir:
        manager = S3ComputeLogManager(
            bucket=mock_s3_bucket.name, prefix=s3_prefix, local_dir=temp_dir, skip_empty_files=True
        )
        log_key_prefix = ["test_log_bucket", evaluation_time.strftime("%Y%m%d_%H%M%S")]

        def write_log_file(file_id: int, io_type: ComputeIOType):
            full_log_key = [*log_key_prefix, f"{file_id}"]
            with manager.open_log_stream(full_log_key, io_type) as f:
                f.write("foo")  # pyright: ignore[reportOptionalMemberAccess]

    log_keys = manager.get_log_keys_for_log_key_prefix(log_key_prefix, io_type=ComputeIOType.STDERR)
    assert len(log_keys) == 0

    for i in range(4):
        write_log_file(i, ComputeIOType.STDERR)

    log_keys = manager.get_log_keys_for_log_key_prefix(log_key_prefix, io_type=ComputeIOType.STDERR)
    assert sorted(log_keys) == [  # pyright: ignore[reportArgumentType]
        [*log_key_prefix, "0"],
        [*log_key_prefix, "1"],
        [*log_key_prefix, "2"],
        [*log_key_prefix, "3"],
    ]

    # write a different file type
    write_log_file(4, ComputeIOType.STDOUT)

    log_keys = manager.get_log_keys_for_log_key_prefix(log_key_prefix, io_type=ComputeIOType.STDERR)
    assert sorted(log_keys) == [  # pyright: ignore[reportArgumentType]
        [*log_key_prefix, "0"],
        [*log_key_prefix, "1"],
        [*log_key_prefix, "2"],
        [*log_key_prefix, "3"],
    ]


class TestS3ComputeLogManager(TestComputeLogManager):
    __test__ = True

    @pytest.fixture(name="compute_log_manager")
    def compute_log_manager(self, mock_s3_bucket):
        with tempfile.TemporaryDirectory() as temp_dir:
            yield S3ComputeLogManager(
                bucket=mock_s3_bucket.name, prefix="my_prefix", local_dir=temp_dir
            )

    # for streaming tests
    @pytest.fixture(name="write_manager")
    def write_manager(self, mock_s3_bucket):
        # should be a different local directory as the read manager
        with tempfile.TemporaryDirectory() as temp_dir:
            yield S3ComputeLogManager(
                bucket=mock_s3_bucket.name,
                prefix="my_prefix",
                local_dir=temp_dir,
                upload_interval=1,
            )

    @pytest.fixture(name="read_manager")
    def read_manager(self, mock_s3_bucket):
        # should be a different local directory as the write manager
        with tempfile.TemporaryDirectory() as temp_dir:
            yield S3ComputeLogManager(
                bucket=mock_s3_bucket.name, prefix="my_prefix", local_dir=temp_dir
            )


def test_external_compute_log_manager(mock_s3_bucket):
    @op
    def my_op():
        print("hello out")  # noqa: T201
        print("hello error", file=sys.stderr)  # noqa: T201

    @job
    def my_job():
        my_op()

    with instance_for_test(
        overrides={
            "compute_logs": {
                "module": "dagster_aws.s3.compute_log_manager",
                "class": "S3ComputeLogManager",
                "config": {
                    "bucket": mock_s3_bucket.name,
                    "show_url_only": True,
                },
            },
        },
    ) as instance:
        result = my_job.execute_in_process(instance=instance)
        assert result.success
        assert result.run_id
        captured_log_entries = instance.all_logs(
            result.run_id, of_type=DagsterEventType.LOGS_CAPTURED
        )
        assert len(captured_log_entries) == 1
        entry = captured_log_entries[0]
        assert entry.dagster_event.logs_captured_data.external_stdout_url  # pyright: ignore[reportOptionalMemberAccess]
        assert entry.dagster_event.logs_captured_data.external_stderr_url  # pyright: ignore[reportOptionalMemberAccess]



import time

import boto3
import pytest
from moto import mock_aws

from dagster_aws.s3.sensor import get_objects, get_s3_keys

BUCKET_NAME = "test-bucket"
PREFIX = "content"


def _put_object(client, bucket_name, prefix, key, body, delay=0) -> None:
    time.sleep(delay)
    r = client.put_object(Bucket=bucket_name, Key=f"{prefix}/{key}", Body=body)
    if not r:
        raise Exception("Failed to create object")


def test_get_s3_keys():
    with mock_aws():
        s3_client = boto3.client("s3", region_name="us-east-1")
        response = s3_client.create_bucket(Bucket=BUCKET_NAME)
        if not response:
            raise Exception("Failed to create bucket")

        no_key = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX)
        assert len(no_key) == 0  # no keys in bucket

        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="foo-1",
            body="test",
        )
        one_key = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX)
        assert len(one_key) == 1

        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="foo-2",
            body="test",
        )
        two_keys = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX)
        assert len(two_keys) == 2, "both keys should be returned"

        one_key_shift = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX, since_key=two_keys[0])
        assert len(one_key_shift) == 1, "only one key should be returned"

        two_key_shift = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX, since_key=two_keys[1])
        assert len(two_key_shift) == 0, "no keys should be returned"

        # test pagination
        for i in range(3, 1002):
            _put_object(
                client=s3_client,
                bucket_name=BUCKET_NAME,
                prefix=PREFIX,
                key=f"foo-{i}",
                body="test",
            )
        thousand_and_one_keys = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX)
        assert len(thousand_and_one_keys) == 1001, "1001 keys should be returned"

        #  test pagination with since_key
        # keys are sorted by LastModified time, with a resolution of 1 second, so the key at index 999 will vary.
        # The test would be flaky if sorted was not guaranteed to be stable, which it is
        # according to: https://wiki.python.org/moin/HowTo/Sorting/.
        keys = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX, since_key=thousand_and_one_keys[999])
        assert len(keys) == 1, "1 key should be returned"

        #  test pagination with since_key > 1000
        for i in range(1002, 1101):
            _put_object(
                client=s3_client,
                bucket_name=BUCKET_NAME,
                prefix=PREFIX,
                key=f"foo-{i}",
                body="test",
            )
        thousand_and_one_hundred_keys = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX)
        assert len(thousand_and_one_hundred_keys) == 1100, "1100 keys should be returned"

        # keys are sorted by LastModified time, with a resolution of 1 second, so the key at index 999 will vary.
        # The test would be flaky if sorted was not guaranteed to be stable, which it is
        # according to: https://wiki.python.org/moin/HowTo/Sorting/.
        keys = get_s3_keys(
            bucket=BUCKET_NAME,
            prefix=PREFIX,
            since_key=thousand_and_one_hundred_keys[1090],
        )
        assert len(keys) == 9, "9 keys should be returned"


def test_get_s3_keys_missing_key_raises_exception():
    with mock_aws():
        s3_client = boto3.client("s3", region_name="us-east-1")
        response = s3_client.create_bucket(Bucket=BUCKET_NAME)
        if not response:
            raise Exception("Failed to create bucket")

        no_key = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX)
        assert len(no_key) == 0  # no keys in bucket

        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="foo-1",
            body="test",
        )
        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="foo-2",
            body="test",
        )
        two_keys = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX)
        assert len(two_keys) == 2

        # if `since_key` is not present in the list of objects, then an exception is thrown
        with pytest.raises(Exception):
            _ = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX, since_key="content/foo-z")


def test_get_s3_since_key_with_modified_files():
    with mock_aws():
        s3_client = boto3.client("s3", region_name="us-east-1")
        s3_client.create_bucket(Bucket=BUCKET_NAME)

        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="A",
            body="initial content",
            delay=1,
        )

        initial_keys = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX)
        assert len(initial_keys) == 1
        assert initial_keys[0].endswith("A")

        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="B",
            body="content B",
            delay=1,
        )
        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="C",
            body="content C",
            delay=1,
        )

        # Modify the original `A` object
        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="A",
            body="modified content",
            delay=1,
        )

        new_keys = get_s3_keys(bucket=BUCKET_NAME, prefix=PREFIX, since_key=initial_keys[0])

        # Since A has been modified, and B and C were created before this key, we will not get new keys
        assert len(new_keys) == 0


def test_get_objects():
    with mock_aws():
        s3_client = boto3.client("s3", region_name="us-east-1")
        s3_client.create_bucket(Bucket=BUCKET_NAME)

        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="A",
            body="initial content",
            delay=1,
        )

        objects = get_objects(bucket=BUCKET_NAME, prefix=PREFIX)

        obj = objects[0]
        object_key = obj.get("Key")
        object_last_modified = obj.get("LastModified")
        assert len(objects) == 1
        assert object_key.endswith("A")

        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="B",
            body="content B",
            delay=1,
        )
        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="C",
            body="content C",
            delay=1,
        )

        new_keys = get_objects(
            bucket=BUCKET_NAME, prefix=PREFIX, since_last_modified=object_last_modified
        )

        assert len(new_keys) == 2


def test_get_objects_with_modified():
    with mock_aws():
        s3_client = boto3.client("s3", region_name="us-east-1")
        s3_client.create_bucket(Bucket=BUCKET_NAME)

        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="A",
            body="content",
            delay=1,
        )

        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="B",
            body="content",
            delay=1,
        )

        objects = get_objects(bucket=BUCKET_NAME, prefix=PREFIX)
        assert len(objects) == 2

        obj = objects[1]
        obj_b_last_modified = obj.get("LastModified")

        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="C",
            body="content",
            delay=1,
        )

        # modify timestamp of B
        _put_object(
            client=s3_client,
            bucket_name=BUCKET_NAME,
            prefix=PREFIX,
            key="B",
            body="updated content",
            delay=1,
        )

        objects = get_objects(
            bucket=BUCKET_NAME, prefix=PREFIX, since_last_modified=obj_b_last_modified
        )

        assert len(objects) == 2
        assert objects[0].get("Key").endswith("C")
        assert objects[1].get("Key").endswith("B")

        # Get objects later than the latest. This must return an empty list.
        objects = get_objects(
            bucket=BUCKET_NAME, prefix=PREFIX, since_last_modified=objects[-1].get("LastModified")
        )

        assert not objects



import multiprocessing
import os
import re
import sys
import time
from typing import TYPE_CHECKING

import boto3
import botocore
import pytest
from dagster import asset, materialize
from dagster._core.definitions.asset_checks.asset_check_spec import AssetCheckKey, AssetCheckSpec
from dagster._core.definitions.data_version import (
    DATA_VERSION_IS_USER_PROVIDED_TAG,
    DATA_VERSION_TAG,
)
from dagster._core.definitions.events import AssetKey
from dagster._core.definitions.metadata import MarkdownMetadataValue
from dagster._core.execution.context.compute import AssetExecutionContext
from dagster._core.instance_for_test import instance_for_test
from dagster._core.storage.asset_check_execution_record import AssetCheckExecutionRecordStatus

from dagster_aws.pipes import PipesCloudWatchMessageReader, PipesECSClient
from dagster_aws.pipes.clients.ecs import WaiterConfig
from dagster_aws_tests.pipes_tests.fake_ecs import LocalECSMockClient
from dagster_aws_tests.pipes_tests.utils import _MOTO_SERVER_URL

if TYPE_CHECKING:
    from mypy_boto3_ecs import ECSClient


@pytest.fixture
def ecs_client(moto_server, s3_client) -> "ECSClient":
    return boto3.client("ecs", region_name="us-east-1", endpoint_url=_MOTO_SERVER_URL)


@pytest.fixture
def ecs_cluster(ecs_client) -> str:
    cluster_name = "test-cluster"
    ecs_client.create_cluster(clusterName=cluster_name)
    return cluster_name


@pytest.fixture
def ecs_task_definition(ecs_client, external_script_default_components) -> str:
    task_definition = "test-task"
    ecs_client.register_task_definition(
        family=task_definition,
        containerDefinitions=[
            {
                "name": LocalECSMockClient.CONTAINER_NAME,
                "image": "test-image",
                "command": [sys.executable, external_script_default_components],
                "memory": 512,
            }
        ],
    )
    return task_definition


@pytest.fixture
def local_ecs_mock_client(
    ecs_client, cloudwatch_client, ecs_cluster, ecs_task_definition
) -> LocalECSMockClient:
    return LocalECSMockClient(ecs_client=ecs_client, cloudwatch_client=cloudwatch_client)


@pytest.fixture
def pipes_ecs_client(local_ecs_mock_client, s3_client, cloudwatch_client) -> PipesECSClient:
    return PipesECSClient(
        client=local_ecs_mock_client,
        message_reader=PipesCloudWatchMessageReader(
            client=cloudwatch_client,
        ),
    )


@asset(check_specs=[AssetCheckSpec(name="foo_check", asset=AssetKey(["ecs_asset"]))])
def ecs_asset(context: AssetExecutionContext, pipes_ecs_client: PipesECSClient):
    return pipes_ecs_client.run(
        context=context,
        extras={"bar": "baz"},
        run_task_params={
            "cluster": "test-cluster",
            "count": 1,
            "taskDefinition": "test-task",
            "launchType": "FARGATE",
            "networkConfiguration": {"awsvpcConfiguration": {"subnets": ["subnet-12345678"]}},
            "overrides": {
                "containerOverrides": [
                    {
                        "name": LocalECSMockClient.CONTAINER_NAME,
                        "environment": [
                            {
                                "name": "SLEEP_SECONDS",
                                "value": os.getenv(
                                    "SLEEP_SECONDS", "0.1"
                                ),  # this can be increased to test interruption
                            }
                        ],
                    }
                ]
            },
        },
        waiter_config=WaiterConfig(
            Delay=int(os.getenv("WAIT_DELAY", "6")),
            MaxAttempts=int(os.getenv("WAIT_MAX_ATTEMPTS", "1000000")),
        ),
    ).get_results()


def test_ecs_pipes(
    capsys,
    pipes_ecs_client: PipesECSClient,
):
    with instance_for_test() as instance:
        materialize(
            [ecs_asset], instance=instance, resources={"pipes_ecs_client": pipes_ecs_client}
        )
        mat = instance.get_latest_materialization_event(ecs_asset.key)
        assert mat and mat.asset_materialization
        assert isinstance(mat.asset_materialization.metadata["bar"], MarkdownMetadataValue)
        assert mat.asset_materialization.metadata["bar"].value == "baz"
        assert "AWS ECS Task URL" in mat.asset_materialization.metadata
        assert mat.asset_materialization.tags
        assert mat.asset_materialization.tags[DATA_VERSION_TAG] == "alpha"
        assert mat.asset_materialization.tags[DATA_VERSION_IS_USER_PROVIDED_TAG]

        captured = capsys.readouterr()
        assert re.search(r"dagster - INFO - [^\n]+ - hello world\n", captured.err, re.MULTILINE)

        asset_check_executions = instance.event_log_storage.get_asset_check_execution_history(
            check_key=AssetCheckKey(ecs_asset.key, name="foo_check"),
            limit=1,
        )
        assert len(asset_check_executions) == 1
        assert asset_check_executions[0].status == AssetCheckExecutionRecordStatus.SUCCEEDED


def _materialize_asset(env, return_dict):
    ecs_client = boto3.client("ecs", region_name="us-east-1", endpoint_url=_MOTO_SERVER_URL)
    cloudwatch_client = boto3.client("logs", region_name="us-east-1", endpoint_url=_MOTO_SERVER_URL)
    pipes = PipesECSClient(
        client=LocalECSMockClient(ecs_client=ecs_client, cloudwatch_client=cloudwatch_client),  # type: ignore
        message_reader=PipesCloudWatchMessageReader(
            client=cloudwatch_client,
        ),
    )
    os.environ.update(env)
    try:
        with instance_for_test() as instance:
            materialize(  # this will be interrupted and raise an exception
                [ecs_asset],
                instance=instance,
                resources={"pipes_ecs_client": pipes},
            )
    finally:
        assert len(pipes._client._task_runs) > 0  # noqa  # pyright: ignore[reportAttributeAccessIssue]
        task_arn = next(iter(pipes._client._task_runs.keys()))  # noqa  # pyright: ignore[reportAttributeAccessIssue]
        return_dict[0] = pipes._client.describe_tasks(  # noqa
            cluster="test-cluster", tasks=[task_arn]
        )


def test_ecs_pipes_interruption_forwarding(pipes_ecs_client: PipesECSClient):
    with multiprocessing.Manager() as manager:
        return_dict = manager.dict()

        p = multiprocessing.Process(
            target=_materialize_asset,
            args=(
                {"SLEEP_SECONDS": "10"},
                return_dict,
            ),
        )
        p.start()

        while p.is_alive():
            # we started executing the run
            # time to interrupt it!
            time.sleep(4)
            p.terminate()

        p.join()
        assert not p.is_alive()
        # breakpoint()
        assert return_dict[0]["tasks"][0]["containers"][0]["exitCode"] == 1
        assert return_dict[0]["tasks"][0]["stoppedReason"] == "Dagster process was interrupted"


class FailToStartECSMockClient(LocalECSMockClient):
    """Mock client that simulates a task failing to start (e.g. image pull failure)."""

    FAIL_REASON = (
        "TaskFailedToStart: CannotPullContainerError: pull image manifest has been retried"
        " 1 time(s): failed to resolve ref docker.io/amazon/aws-for-fluent-bit:stable"
    )

    def run_task(self, **kwargs):
        response = super().run_task(**kwargs)
        task_arn = response["tasks"][0]["taskArn"]
        self.simulate_task_failed_to_start(task_arn, self.FAIL_REASON)
        return response


def test_ecs_pipes_task_failed_to_start(
    ecs_client, cloudwatch_client, ecs_cluster, ecs_task_definition
):
    """Test that a task which fails to start (e.g. image pull failure, missing IAM permissions)
    raises an error instead of producing a successful materialization.
    """
    mock_client = FailToStartECSMockClient(
        ecs_client=ecs_client, cloudwatch_client=cloudwatch_client
    )
    pipes = PipesECSClient(
        client=mock_client,  # type: ignore
        message_reader=PipesCloudWatchMessageReader(client=cloudwatch_client),
    )
    with instance_for_test() as instance:
        with pytest.raises(RuntimeError, match="ECS task failed to start"):
            materialize([ecs_asset], instance=instance, resources={"pipes_ecs_client": pipes})


def test_ecs_pipes_waiter_config(pipes_ecs_client: PipesECSClient):
    with instance_for_test() as instance:
        """
        Test Error is thrown when the wait delay is less than the processing time.
        """
        os.environ.update({"WAIT_DELAY": "1", "WAIT_MAX_ATTEMPTS": "1", "SLEEP_SECONDS": "2"})
        with pytest.raises(botocore.exceptions.WaiterError, match=r".* Max attempts exceeded"):  # pyright: ignore (reportAttributeAccessIssue)
            materialize(
                [ecs_asset], instance=instance, resources={"pipes_ecs_client": pipes_ecs_client}
            )

        """
        Test Error is thrown when the wait attempts * wait delay is less than the processing time.
        """
        os.environ.update({"WAIT_DELAY": "1", "WAIT_MAX_ATTEMPTS": "2", "SLEEP_SECONDS": "3"})
        with pytest.raises(botocore.exceptions.WaiterError, match=r".* Max attempts exceeded"):  # pyright: ignore (reportAttributeAccessIssue)
            materialize(
                [ecs_asset], instance=instance, resources={"pipes_ecs_client": pipes_ecs_client}
            )

        """
        Test asset is materialized successfully when the wait attempts * wait delay is greater than the processing time.
        """
        os.environ.update({"WAIT_DELAY": "1", "WAIT_MAX_ATTEMPTS": "10", "SLEEP_SECONDS": "2"})
        materialize(
            [ecs_asset], instance=instance, resources={"pipes_ecs_client": pipes_ecs_client}
        )
        mat = instance.get_latest_materialization_event(ecs_asset.key)
        assert mat and mat.asset_materialization



import multiprocessing
import os
import re
import signal
import time
from collections.abc import Iterator
from typing import Literal

import boto3
import pytest
from dagster import AssetsDefinition, asset, materialize
from dagster._core.definitions.asset_checks.asset_check_spec import AssetCheckKey, AssetCheckSpec
from dagster._core.definitions.data_version import (
    DATA_VERSION_IS_USER_PROVIDED_TAG,
    DATA_VERSION_TAG,
)
from dagster._core.definitions.events import AssetKey
from dagster._core.definitions.metadata import MarkdownMetadataValue
from dagster._core.execution.context.compute import AssetExecutionContext
from dagster._core.instance_for_test import instance_for_test
from dagster._core.storage.asset_check_execution_record import AssetCheckExecutionRecordStatus

from dagster_aws.pipes import (
    PipesCloudWatchMessageReader,
    PipesGlueClient,
    PipesS3ContextInjector,
    PipesS3MessageReader,
)
from dagster_aws_tests.pipes_tests.fake_glue import LocalGlueMockClient
from dagster_aws_tests.pipes_tests.utils import _MOTO_SERVER_URL, _S3_TEST_BUCKET, temp_script

GLUE_JOB_NAME = "test-job"


@pytest.fixture
def external_s3_glue_script(s3_client) -> Iterator[str]:
    # This is called in an external process and so cannot access outer scope
    def script_fn():
        import os
        import time

        import boto3
        from dagster_pipes import (
            PipesCliArgsParamsLoader,
            PipesS3ContextLoader,
            PipesS3MessageWriter,
            open_dagster_pipes,
        )

        s3_client = boto3.client(
            "s3", region_name="us-east-1", endpoint_url="http://localhost:5193"
        )

        messages_backend = os.environ["TESTING_PIPES_MESSAGES_BACKEND"]
        if messages_backend == "s3":
            message_writer = PipesS3MessageWriter(s3_client, interval=0.001)
        else:
            message_writer = None

        with open_dagster_pipes(
            context_loader=PipesS3ContextLoader(client=s3_client),
            message_writer=message_writer,
            params_loader=PipesCliArgsParamsLoader(),
        ) as context:
            context.log.info("hello world")
            time.sleep(0.1)  # sleep to make sure that we encompass multiple intervals for S3 IO
            context.report_asset_materialization(
                metadata={"bar": {"raw_value": context.get_extra("bar"), "type": "md"}},
                data_version="alpha",
            )
            context.report_asset_check(
                "foo_check",
                passed=True,
                severity="WARN",
                metadata={
                    "meta_1": 1,
                    "meta_2": {"raw_value": "foo", "type": "text"},
                },
            )

    with temp_script(script_fn) as script_path:
        s3_client.upload_file(script_path, _S3_TEST_BUCKET, "glue_script.py")
        # yield path on s3
        yield f"s3://{_S3_TEST_BUCKET}/glue_script.py"


@pytest.fixture
def glue_client(moto_server, external_s3_glue_script, s3_client):
    client = boto3.client("glue", region_name="us-east-1", endpoint_url=_MOTO_SERVER_URL)
    client.create_job(
        Name=GLUE_JOB_NAME,
        Description="Test job",
        Command={
            "Name": "glueetl",  # Spark job type
            "ScriptLocation": external_s3_glue_script,
            "PythonVersion": "3.10",
        },
        GlueVersion="4.0",
        Role="arn:aws:iam::012345678901:role/service-role/AWSGlueServiceRole-test",
    )
    return client


@pytest.fixture
def long_glue_job(s3_client, glue_client) -> Iterator[str]:
    job_name = "Very Long Job"

    def script_fn():
        import os
        import time

        import boto3
        from dagster_pipes import PipesCliArgsParamsLoader, PipesS3ContextLoader, open_dagster_pipes

        s3_client = boto3.client(
            "s3", region_name="us-east-1", endpoint_url="http://localhost:5193"
        )

        with open_dagster_pipes(
            context_loader=PipesS3ContextLoader(client=s3_client),
            params_loader=PipesCliArgsParamsLoader(),
        ) as context:
            context.log.info("Glue job sleeping...")
            time.sleep(int(os.getenv("SLEEP_SECONDS", "1")))

    with temp_script(script_fn) as script_path:
        s3_key = "long_glue_script.py"
        s3_client.upload_file(script_path, _S3_TEST_BUCKET, s3_key)

        glue_client.create_job(
            Name=job_name,
            Description="Test job",
            Command={
                "Name": "glueetl",  # Spark job type
                "ScriptLocation": f"s3://{_S3_TEST_BUCKET}/{s3_key}",
                "PythonVersion": "3.10",
            },
            GlueVersion="4.0",
            Role="arn:aws:iam::012345678901:role/service-role/AWSGlueServiceRole-test",
        )

        yield job_name


@pytest.fixture
def glue_asset(long_glue_job: str) -> AssetsDefinition:
    @asset
    def foo(context: AssetExecutionContext, pipes_glue_client: PipesGlueClient):
        results = pipes_glue_client.run(
            context=context,
            start_job_run_params={"JobName": long_glue_job},
        ).get_results()
        return results

    return foo


@pytest.fixture
def local_glue_mock_client(glue_client, s3_client, cloudwatch_client) -> LocalGlueMockClient:
    return LocalGlueMockClient(
        aws_endpoint_url=_MOTO_SERVER_URL,
        glue_client=glue_client,
        s3_client=s3_client,
        cloudwatch_client=cloudwatch_client,
        pipes_messages_backend="cloudwatch",
    )


@pytest.fixture
def pipes_glue_client(local_glue_mock_client, s3_client, cloudwatch_client) -> PipesGlueClient:
    return PipesGlueClient(
        client=local_glue_mock_client,
        context_injector=PipesS3ContextInjector(bucket=_S3_TEST_BUCKET, client=s3_client),
        message_reader=PipesCloudWatchMessageReader(client=cloudwatch_client),
    )


@pytest.mark.parametrize("pipes_messages_backend", ["s3", "cloudwatch"])
def test_glue_pipes(
    capsys,
    s3_client,
    glue_client,
    cloudwatch_client,
    pipes_messages_backend: Literal["s3", "cloudwatch"],
):
    @asset(check_specs=[AssetCheckSpec(name="foo_check", asset=AssetKey(["foo"]))])
    def foo(context: AssetExecutionContext, pipes_glue_client: PipesGlueClient):
        results = pipes_glue_client.run(
            context=context,
            start_job_run_params={"JobName": GLUE_JOB_NAME},
            extras={"bar": "baz"},
        ).get_results()
        return results

    context_injector = PipesS3ContextInjector(bucket=_S3_TEST_BUCKET, client=s3_client)
    message_reader = (
        PipesS3MessageReader(bucket=_S3_TEST_BUCKET, client=s3_client, interval=0.001)
        if pipes_messages_backend == "s3"
        else PipesCloudWatchMessageReader(client=cloudwatch_client)
    )

    pipes_glue_client = PipesGlueClient(
        client=LocalGlueMockClient(  # pyright: ignore[reportArgumentType]
            aws_endpoint_url=_MOTO_SERVER_URL,
            glue_client=glue_client,
            s3_client=s3_client,
            cloudwatch_client=cloudwatch_client,
            pipes_messages_backend=pipes_messages_backend,
        ),
        context_injector=context_injector,
        message_reader=message_reader,
    )

    with instance_for_test() as instance:
        materialize([foo], instance=instance, resources={"pipes_glue_client": pipes_glue_client})
        mat = instance.get_latest_materialization_event(foo.key)
        assert mat and mat.asset_materialization
        assert isinstance(mat.asset_materialization.metadata["bar"], MarkdownMetadataValue)
        assert mat.asset_materialization.metadata["bar"].value == "baz"
        assert "AWS Glue Job Run ID" in mat.asset_materialization.metadata
        assert mat.asset_materialization.tags
        assert mat.asset_materialization.tags[DATA_VERSION_TAG] == "alpha"
        assert mat.asset_materialization.tags[DATA_VERSION_IS_USER_PROVIDED_TAG]

        captured = capsys.readouterr()
        assert re.search(r"dagster - INFO - [^\n]+ - hello world\n", captured.err, re.MULTILINE)

        asset_check_executions = instance.event_log_storage.get_asset_check_execution_history(
            check_key=AssetCheckKey(foo.key, name="foo_check"),
            limit=1,
        )
        assert len(asset_check_executions) == 1
        assert asset_check_executions[0].status == AssetCheckExecutionRecordStatus.SUCCEEDED


def test_glue_pipes_interruption_forwarding_asset_is_valid(glue_asset, pipes_glue_client):
    # make sure this runs without multiprocessing first

    with instance_for_test() as instance:
        materialize(
            [glue_asset], instance=instance, resources={"pipes_glue_client": pipes_glue_client}
        )


def test_glue_pipes_interruption_forwarding(long_glue_job, glue_asset, pipes_glue_client):
    def materialize_asset(env, return_dict):
        os.environ.update(env)
        try:
            with instance_for_test() as instance:
                materialize(  # this will be interrupted and raise an exception
                    [glue_asset],
                    instance=instance,
                    resources={"pipes_glue_client": pipes_glue_client},
                )
        finally:
            job_runs = pipes_glue_client._client._job_runs  # noqa
            if job_runs:
                job_run_id = next(iter(job_runs.keys()))
                return_dict[0] = pipes_glue_client._client.get_job_run(long_glue_job, job_run_id)  # noqa

    # Use fork context explicitly - Python 3.14 changed the default to spawn on macOS,
    # which requires pickling the target function (local functions can't be pickled)
    ctx = multiprocessing.get_context("fork")
    with ctx.Manager() as manager:
        return_dict = manager.dict()

        p = ctx.Process(
            target=materialize_asset,
            args=(
                {"SLEEP_SECONDS": "10"},
                return_dict,
            ),
        )
        p.start()

        while p.is_alive():
            # we started executing the run
            # time to interrupt it!
            time.sleep(5)
            # Use SIGINT instead of p.terminate() (SIGTERM) so that Python
            # raises KeyboardInterrupt and unwinds the stack normally,
            # allowing the finally block to execute reliably.
            assert p.pid is not None
            try:
                os.kill(p.pid, signal.SIGINT)
            except ProcessLookupError:
                # Process already terminated - this is acceptable for this test
                pass

        p.join()
        assert not p.is_alive()
        assert 0 in return_dict, (
            "Interrupt arrived before the Glue job started; the finally block had nothing to report"
        )
        assert return_dict[0]["JobRun"]["JobRunState"] == "STOPPED"



from unittest.mock import MagicMock, patch

import pytest
from dagster._core.test_utils import environ


def test_secrets(
    ecs,
    secrets_manager,
    instance_cm,
    launch_run,
    tagged_secret,
    other_secret,
    configured_secret,
):
    initial_task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]

    config = {
        "secrets": [
            {
                "name": "HELLO",
                "valueFrom": configured_secret.arn + "/hello",
            }
        ],
    }

    with instance_cm(config) as instance:
        launch_run(instance)

    # A new task definition is created
    task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]
    assert len(task_definitions) == len(initial_task_definitions) + 1
    task_definition_arn = next(iter(set(task_definitions).difference(initial_task_definitions)))
    task_definition = ecs.describe_task_definition(taskDefinition=task_definition_arn)
    task_definition = task_definition["taskDefinition"]

    # It includes tagged secrets
    secrets = task_definition["containerDefinitions"][0]["secrets"]
    assert {"name": tagged_secret.name, "valueFrom": tagged_secret.arn} in secrets

    # And configured secrets
    assert {
        "name": "HELLO",
        "valueFrom": configured_secret.arn + "/hello",
    } in secrets

    # But no other secrets
    assert len(secrets) == 2


def test_environment(
    ecs,
    instance_cm,
    launch_run,
):
    initial_task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]

    config = {"env_vars": ["FOO_ENV_VAR=BAR_VALUE"]}

    with instance_cm(config) as instance:
        launch_run(instance)

    # A new task definition is created
    task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]
    assert len(task_definitions) == len(initial_task_definitions) + 1
    task_definition_arn = next(iter(set(task_definitions).difference(initial_task_definitions)))
    task_definition = ecs.describe_task_definition(taskDefinition=task_definition_arn)
    task_definition = task_definition["taskDefinition"]

    # It includes the environment
    environment = task_definition["containerDefinitions"][0]["environment"]
    assert {"name": "FOO_ENV_VAR", "value": "BAR_VALUE"} in environment


def test_environment_missing_env_var_value(
    ecs,
    instance_cm,
    launch_run,
):
    initial_task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]

    config = {"env_vars": ["FOO_ENV_VAR"]}

    with instance_cm(config) as instance:
        with pytest.raises(
            Exception, match="Tried to load environment variable FOO_ENV_VAR, but it was not set"
        ):
            launch_run(instance)

        with environ({"FOO_ENV_VAR": "BAR_VALUE"}):
            launch_run(instance)

    # A new task definition is created
    task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]
    assert len(task_definitions) == len(initial_task_definitions) + 1
    task_definition_arn = next(iter(set(task_definitions).difference(initial_task_definitions)))
    task_definition = ecs.describe_task_definition(taskDefinition=task_definition_arn)
    task_definition = task_definition["taskDefinition"]

    # It includes the environment
    environment = task_definition["containerDefinitions"][0]["environment"]
    assert {"name": "FOO_ENV_VAR", "value": "BAR_VALUE"} in environment


def test_secrets_with_container_context(
    ecs,
    secrets_manager,
    instance_cm,
    launch_run_with_container_context,
    tagged_secret,
    other_secret,
    configured_secret,
):
    initial_task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]

    # Secrets config is pulled from container context on the run, rather than run launcher config
    config = {"secrets_tag": None, "secrets": []}

    with instance_cm(config) as instance:
        launch_run_with_container_context(instance)

    # A new task definition is created
    task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]
    assert len(task_definitions) == len(initial_task_definitions) + 1
    task_definition_arn = next(iter(set(task_definitions).difference(initial_task_definitions)))
    task_definition = ecs.describe_task_definition(taskDefinition=task_definition_arn)
    task_definition = task_definition["taskDefinition"]

    # It includes tagged secrets
    secrets = task_definition["containerDefinitions"][0]["secrets"]
    assert {"name": tagged_secret.name, "valueFrom": tagged_secret.arn} in secrets

    # And configured secrets
    assert {
        "name": "HELLO",
        "valueFrom": configured_secret.arn + "/hello",
    } in secrets

    # But no other secrets
    assert len(secrets) == 2


def test_environment_with_container_context(
    ecs,
    instance_cm,
    launch_run_with_container_context,
):
    initial_task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]

    # Secrets config is pulled from container context on the run, rather than run launcher config
    config = {"env_vars": ["RUN_LAUNCHER_NAME=RUN_LAUNCHER_VALUE"]}

    with instance_cm(config) as instance:
        launch_run_with_container_context(instance)

    # A new task definition is created
    task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]
    assert len(task_definitions) == len(initial_task_definitions) + 1
    task_definition_arn = next(iter(set(task_definitions).difference(initial_task_definitions)))
    task_definition = ecs.describe_task_definition(taskDefinition=task_definition_arn)
    task_definition = task_definition["taskDefinition"]

    # It includes environment from run launcher
    environment = task_definition["containerDefinitions"][0]["environment"]

    assert {"name": "RUN_LAUNCHER_NAME", "value": "RUN_LAUNCHER_VALUE"} in environment

    # And container context
    assert {
        "name": "FOO_ENV_VAR",
        "value": "BAR_VALUE",
    } in environment


def test_secrets_backcompat(
    ecs,
    secrets_manager,
    instance_cm,
    launch_run,
    tagged_secret,
    other_secret,
    configured_secret,
):
    initial_task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]

    with pytest.warns(DeprecationWarning, match="Setting secrets as a list of ARNs is deprecated"):
        with instance_cm({"secrets": [configured_secret.arn]}) as instance:
            launch_run(instance)

    # A new task definition is created
    task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]
    assert len(task_definitions) == len(initial_task_definitions) + 1
    task_definition_arn = next(iter(set(task_definitions).difference(initial_task_definitions)))
    task_definition = ecs.describe_task_definition(taskDefinition=task_definition_arn)
    task_definition = task_definition["taskDefinition"]

    # It includes tagged secrets
    secrets = task_definition["containerDefinitions"][0]["secrets"]
    assert {"name": tagged_secret.name, "valueFrom": tagged_secret.arn} in secrets

    # And configured secrets
    assert {"name": configured_secret.name, "valueFrom": configured_secret.arn} in secrets

    # But no other secrets
    assert len(secrets) == 2


def test_empty_secrets(
    ecs,
    secrets_manager,
    instance_cm,
    launch_run,
):
    initial_task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]

    with instance_cm({"secrets_tag": None}) as instance:
        m = MagicMock()
        with patch.object(instance.run_launcher, "secrets_manager", new=m):
            launch_run(instance)

        m.get_paginator.assert_not_called()
        m.describe_secret.assert_not_called()

    # A new task definition is created
    task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]
    assert len(task_definitions) == len(initial_task_definitions) + 1
    task_definition_arn = next(iter(set(task_definitions).difference(initial_task_definitions)))
    task_definition = ecs.describe_task_definition(taskDefinition=task_definition_arn)
    task_definition = task_definition["taskDefinition"]

    # No secrets
    assert not task_definition["containerDefinitions"][0].get("secrets")



import json
from collections import namedtuple
from collections.abc import Iterator, Mapping, Sequence
from contextlib import contextmanager
from typing import Any, Callable, ContextManager  # noqa: UP035

import boto3
import pytest
from dagster._core.definitions.job_definition import JobDefinition
from dagster._core.instance import DagsterInstance
from dagster._core.remote_representation.external import RemoteJob
from dagster._core.storage.dagster_run import DagsterRun
from dagster._core.test_utils import in_process_test_workspace, instance_for_test
from dagster._core.types.loadable_target_origin import LoadableTargetOrigin
from dagster._core.workspace.context import WorkspaceRequestContext
from dagster._utils.warnings import disable_dagster_warnings
from moto import mock_aws

from dagster_aws_tests.ecs_tests.launcher_tests import repo

Secret = namedtuple("Secret", ["name", "arn"])


@pytest.fixture(autouse=True)
def ignore_dagster_warnings() -> Iterator[None]:
    with disable_dagster_warnings():
        yield


@pytest.fixture
def cloudwatch_client(region: str):
    with mock_aws():
        yield boto3.client("logs", region_name=region)


@pytest.fixture
def log_group(cloudwatch_client) -> str:
    name = "/dagster-test/test-cloudwatch-logging"
    cloudwatch_client.create_log_group(logGroupName=name)
    return name


@pytest.fixture
def image() -> str:
    return "dagster:first"


@pytest.fixture
def other_image() -> str:
    return "dagster:second"


@pytest.fixture
def environment() -> Sequence[Mapping[str, str]]:
    return [{"name": "FOO", "value": "bar"}]


@pytest.fixture
def task_definition(ecs, image, environment):
    return ecs.register_task_definition(
        family="dagster",
        containerDefinitions=[
            {
                "name": "dagster",
                "image": image,
                "environment": environment,
                "entryPoint": ["ls"],
                "dependsOn": [
                    {
                        "containerName": "other",
                        "condition": "SUCCESS",
                    },
                ],
                "healthCheck": {"command": ["HELLO"]},
            },
            {
                "name": "other",
                "image": image,
                "entryPoint": ["ls"],
            },
        ],
        networkMode="awsvpc",
        memory="512",
        cpu="256",
    )["taskDefinition"]


@pytest.fixture
def assign_public_ip():
    return True


@pytest.fixture
def task(ecs, subnet, security_group, task_definition, assign_public_ip):
    return ecs.run_task(
        taskDefinition=task_definition["family"],
        networkConfiguration={
            "awsvpcConfiguration": {
                "assignPublicIp": "ENABLED" if assign_public_ip else "DISABLED",
                "subnets": [subnet.id],
                "securityGroups": [security_group.id],
            },
        },
    )["tasks"][0]


@pytest.fixture
def stub_aws(ecs, ec2, secrets_manager, cloudwatch_client, monkeypatch):
    def mock_client(*args, **kwargs):
        if "ecs" in args:
            return ecs
        if "secretsmanager" in args:
            return secrets_manager
        if "logs" in args:
            return cloudwatch_client
        else:
            raise Exception("Unexpected args")

    monkeypatch.setattr(boto3, "client", mock_client)
    monkeypatch.setattr(boto3, "resource", lambda *args, **kwargs: ec2)


@pytest.fixture
def stub_ecs_metadata(task, monkeypatch, requests_mock):
    container_uri = "http://metadata_host"
    monkeypatch.setenv("ECS_CONTAINER_METADATA_URI_V4", container_uri)
    container = task["containers"][0]["name"]
    requests_mock.get(container_uri, json={"Name": container})

    task_uri = container_uri + "/task"
    requests_mock.get(
        task_uri,
        json={
            "Cluster": task["clusterArn"],
            "TaskARN": task["taskArn"],
        },
    )


@pytest.fixture
def instance_cm(stub_aws, stub_ecs_metadata) -> Callable[..., ContextManager[DagsterInstance]]:
    @contextmanager
    def cm(config=None):
        overrides = {
            "run_launcher": {
                "module": "dagster_aws.ecs",
                "class": "EcsRunLauncher",
                "config": {**(config or {})},
            }
        }
        with instance_for_test(overrides) as dagster_instance:
            yield dagster_instance

    return cm


@pytest.fixture
def instance(
    instance_cm: Callable[..., ContextManager[DagsterInstance]],
) -> Iterator[DagsterInstance]:
    with instance_cm(
        {
            "run_ecs_tags": [
                {"key": "HAS_VALUE", "value": "SEE"},
                {"key": "DOES_NOT_HAVE_VALUE"},
            ]
        }
    ) as dagster_instance:
        yield dagster_instance


@pytest.fixture
def instance_with_log_group(
    instance_cm: Callable[..., ContextManager[DagsterInstance]], log_group: str
) -> Iterator[DagsterInstance]:
    with instance_cm(config={"task_definition": {"log_group": log_group}}) as dagster_instance:
        yield dagster_instance


@pytest.fixture
def instance_with_resources(
    instance_cm: Callable[..., ContextManager[DagsterInstance]],
) -> Iterator[DagsterInstance]:
    with instance_cm(
        config={
            "run_resources": {
                "cpu": "1024",
                "memory": "2048",
                "ephemeral_storage": 50,
            }
        }
    ) as dagster_instance:
        yield dagster_instance


@pytest.fixture
def instance_dont_use_current_task(
    instance_cm: Callable[..., ContextManager[DagsterInstance]], subnet, monkeypatch
) -> Iterator[DagsterInstance]:
    with instance_cm(
        config={
            "use_current_ecs_task_config": False,
            "run_task_kwargs": {
                "cluster": "my_cluster",
                "networkConfiguration": {
                    "awsvpcConfiguration": {
                        "subnets": [subnet.id],
                        "assignPublicIp": "ENABLED",
                    },
                },
            },
        }
    ) as dagster_instance:
        # Not running in an ECS task
        monkeypatch.setenv("ECS_CONTAINER_METADATA_URI_V4", None)
        yield dagster_instance


@pytest.fixture
def instance_fargate_spot(
    instance_cm: Callable[..., ContextManager[DagsterInstance]],
) -> Iterator[DagsterInstance]:
    with instance_cm(
        config={
            "run_task_kwargs": {
                "capacityProviderStrategy": [
                    {
                        "capacityProvider": "FARGATE_SPOT",
                    }
                ],
            },
        }
    ) as dagster_instance:
        yield dagster_instance


@pytest.fixture
def workspace(instance: DagsterInstance, image: str) -> Iterator[WorkspaceRequestContext]:
    with in_process_test_workspace(
        instance,
        loadable_target_origin=LoadableTargetOrigin(
            python_file=repo.__file__,
            attribute=repo.repository.name,
        ),
        container_image=image,
    ) as workspace:
        yield workspace


@pytest.fixture
def other_workspace(
    instance: DagsterInstance, other_image: str
) -> Iterator[WorkspaceRequestContext]:
    with in_process_test_workspace(
        instance,
        loadable_target_origin=LoadableTargetOrigin(
            python_file=repo.__file__,
            attribute=repo.repository.name,
        ),
        container_image=other_image,
    ) as workspace:
        yield workspace


@pytest.fixture
def job() -> JobDefinition:
    return repo.job


@pytest.fixture
def remote_job(workspace: WorkspaceRequestContext) -> RemoteJob:
    location = workspace.get_code_location(workspace.code_location_names[0])
    return location.get_repository(repo.repository.name).get_full_job(repo.job.name)


@pytest.fixture
def other_remote_job(other_workspace: WorkspaceRequestContext) -> RemoteJob:
    location = other_workspace.get_code_location(other_workspace.code_location_names[0])
    return location.get_repository(repo.repository.name).get_full_job(repo.job.name)


@pytest.fixture
def run(instance: DagsterInstance, job: JobDefinition, remote_job: RemoteJob) -> DagsterRun:
    return instance.create_run_for_job(
        job,
        remote_job_origin=remote_job.get_remote_origin(),
        job_code_origin=remote_job.get_python_origin(),
    )


@pytest.fixture
def other_run(
    instance: DagsterInstance, job: JobDefinition, other_remote_job: RemoteJob
) -> DagsterRun:
    return instance.create_run_for_job(
        job,
        remote_job_origin=other_remote_job.get_remote_origin(),
        job_code_origin=other_remote_job.get_python_origin(),
    )


@pytest.fixture
def launch_run(
    workspace: WorkspaceRequestContext, job: JobDefinition, remote_job: RemoteJob
) -> Callable[[DagsterInstance], None]:
    def _launch_run(instance: DagsterInstance) -> None:
        run = instance.create_run_for_job(
            job,
            remote_job_origin=remote_job.get_remote_origin(),
            job_code_origin=remote_job.get_python_origin(),
        )
        instance.launch_run(run.run_id, workspace)

    return _launch_run


@pytest.fixture
def custom_instance_cm(stub_aws, stub_ecs_metadata):
    @contextmanager
    def cm(config=None):
        overrides = {
            "run_launcher": {
                "module": "dagster_aws.ecs.test_utils",
                "class": "CustomECSRunLauncher",
                "config": {**(config or {})},
            }
        }
        with instance_for_test(overrides) as dagster_instance:
            yield dagster_instance

    return cm


@pytest.fixture
def custom_instance(
    custom_instance_cm: Callable[..., ContextManager[DagsterInstance]],
) -> Iterator[DagsterInstance]:
    with custom_instance_cm() as dagster_instance:
        yield dagster_instance


@pytest.fixture
def custom_workspace(
    custom_instance: DagsterInstance, image: str
) -> Iterator[WorkspaceRequestContext]:
    with in_process_test_workspace(
        custom_instance,
        loadable_target_origin=LoadableTargetOrigin(
            python_file=repo.__file__,
            attribute=repo.repository.name,
        ),
        container_image=image,
    ) as workspace:
        yield workspace


@pytest.fixture
def custom_run(
    custom_instance: DagsterInstance, job: JobDefinition, remote_job: RemoteJob
) -> DagsterRun:
    return custom_instance.create_run_for_job(
        job,
        remote_job_origin=remote_job.get_remote_origin(),
        job_code_origin=remote_job.get_python_origin(),
    )


@pytest.fixture
def tagged_secret(secrets_manager):
    # A secret tagged with "dagster"
    name = "tagged_secret"
    arn = secrets_manager.create_secret(
        Name=name,
        SecretString="hello",
        Tags=[{"Key": "dagster", "Value": "true"}],
    )["ARN"]

    yield Secret(name, arn)


@pytest.fixture
def other_secret(secrets_manager):
    # A secret without a tag
    name = "other_secret"
    arn = secrets_manager.create_secret(
        Name=name,
        SecretString="hello",
    )["ARN"]

    yield Secret(name, arn)


@pytest.fixture
def configured_secret(secrets_manager) -> Iterator[Secret]:
    name = "configured_secret"
    arn = secrets_manager.create_secret(
        Name=name,
        SecretString=json.dumps({"hello": "world"}),
    )["ARN"]

    yield Secret(name, arn)


@pytest.fixture
def other_configured_secret(secrets_manager) -> Iterator[Secret]:
    name = "other_configured_secret"
    arn = secrets_manager.create_secret(
        Name=name,
        SecretString=json.dumps({"goodnight": "moon"}),
    )["ARN"]

    yield Secret(name, arn)


@pytest.fixture
def container_context_config(configured_secret: Secret) -> Mapping[str, Any]:
    return {
        "env_vars": ["SHARED_KEY=SHARED_VAL"],
        "ecs": {
            "secrets": [
                {
                    "name": "HELLO",
                    "valueFrom": configured_secret.arn + "/hello",
                }
            ],
            "secrets_tags": ["dagster"],
            "env_vars": ["FOO_ENV_VAR=BAR_VALUE"],
            "container_name": "foo",
            "run_resources": {
                "cpu": "4096",
                "memory": "8192",
                "ephemeral_storage": 100,
            },
            "server_resources": {
                "cpu": "1024",
                "memory": "2048",
                "ephemeral_storage": 25,
            },
            "task_role_arn": "fake-task-role",
            "execution_role_arn": "fake-execution-role",
            "runtime_platform": {
                "operatingSystemFamily": "WINDOWS_SERVER_2019_FULL",
            },
            "mount_points": [
                {
                    "sourceVolume": "myEfsVolume",
                    "containerPath": "/mount/efs",
                    "readOnly": True,
                }
            ],
            "volumes": [
                {
                    "name": "myEfsVolume",
                    "efsVolumeConfiguration": {
                        "fileSystemId": "fs-1234",
                        "rootDirectory": "/path/to/my/data",
                    },
                }
            ],
            "server_sidecar_containers": [
                {
                    "name": "DatadogAgent",
                    "image": "public.ecr.aws/datadog/agent:latest",
                    "environment": [
                        {"name": "ECS_FARGATE", "value": "true"},
                    ],
                }
            ],
            "run_sidecar_containers": [
                {
                    "name": "busyrun",
                    "image": "busybox:latest",
                }
            ],
            "server_ecs_tags": [
                {
                    "key": "FOO",  # no value
                }
            ],
            "run_ecs_tags": [{"key": "ABC", "value": "DEF"}],  # with value
            "repository_credentials": "fake-secret-arn",
            "server_health_check": {
                "command": ["HELLO"],
            },
        },
    }


@pytest.fixture
def other_container_context_config(other_configured_secret):
    return {
        "env_vars": ["SHARED_OTHER_KEY=SHARED_OTHER_VAL"],
        "ecs": {
            "secrets": [
                {
                    "name": "GOODBYE",
                    "valueFrom": other_configured_secret.arn + "/goodbye",
                }
            ],
            "secrets_tags": ["other_secret_tag"],
            "env_vars": ["OTHER_FOO_ENV_VAR"],
            "container_name": "bar",
            "run_resources": {
                "cpu": "256",
            },
            "server_resources": {
                "cpu": "2048",
                "memory": "4096",
                "replica_count": 2,
            },
            "task_role_arn": "other-task-role",
            "execution_role_arn": "other-fake-execution-role",
            "mount_points": [
                {
                    "sourceVolume": "myOtherEfsVolume",
                    "containerPath": "/mount/other/efs",
                    "readOnly": True,
                }
            ],
            "volumes": [
                {
                    "name": "myOtherEfsVolume",
                    "efsVolumeConfiguration": {
                        "fileSystemId": "fs-5678",
                        "rootDirectory": "/path/to/my/other/data",
                    },
                }
            ],
            "server_sidecar_containers": [
                {
                    "name": "OtherServerAgent",
                    "image": "public.ecr.aws/other/agent:latest",
                }
            ],
            "run_sidecar_containers": [
                {
                    "name": "OtherRunAgent",
                    "image": "otherrun:latest",
                }
            ],
            "server_ecs_tags": [{"key": "BAZ", "value": "QUUX"}],
            "run_ecs_tags": [
                {
                    "key": "GHI",
                }
            ],
            "repository_credentials": "fake-secret-arn",
            "server_health_check": {
                "command": ["CMD-SHELL", "curl -f http://localhost/ || exit 1"],
                "interval": 30,
                "timeout": 5,
                "retries": 3,
                "startPeriod": 0,
            },
        },
    }


@pytest.fixture
def launch_run_with_container_context(
    job: JobDefinition,
    remote_job: RemoteJob,
    workspace: WorkspaceRequestContext,
    container_context_config,
):
    def _launch_run(instance):
        python_origin = remote_job.get_python_origin()
        python_origin = python_origin._replace(
            repository_origin=python_origin.repository_origin._replace(
                container_context=container_context_config,
            )
        )

        run = instance.create_run_for_job(
            job,
            remote_job_origin=remote_job.get_remote_origin(),
            job_code_origin=python_origin,
        )
        instance.launch_run(run.run_id, workspace)

    return _launch_run



import pytest
from dagster._core.test_utils import environ


def test_default_instance(instance_cm):
    with instance_cm() as instance:
        assert instance.run_launcher.use_current_ecs_task_config
        assert instance.run_launcher.run_task_kwargs == {}


def test_run_task_kwargs(instance_cm):
    run_task_kwargs = {
        "launchType": "EC2",
    }

    with instance_cm(
        config={
            "run_task_kwargs": run_task_kwargs,
        }
    ) as instance:
        assert instance.run_launcher.run_task_kwargs == run_task_kwargs


def test_invalid_kwargs_field(instance_cm):
    with pytest.raises(Exception, match="Found an unexpected key foo in run_task_kwargs"):
        with instance_cm(config={"run_task_kwargs": {"foo": "bar"}}) as instance:
            print(instance.run_launcher)  # noqa: T201

    with pytest.raises(Exception):
        with instance_cm(
            config={"taskDefinition": "my-task-def"},
            match="Use the `taskDefinition` config field to pass in a task definition to run",
        ) as instance:
            print(instance.run_launcher)  # noqa: T201

    with pytest.raises(Exception):
        with instance_cm(
            config={"overrides": {"containerOverrides": {}}},
            match=(
                "Task overrides are set by the run launcher and cannot be set in run_task_kwargs."
            ),
        ) as instance:
            print(instance.run_launcher)  # noqa: T201


def test_task_definition_config(instance_cm, task_definition):
    with instance_cm(
        config={"task_definition": "dagster", "container_name": "dagster"}
    ) as instance:
        assert instance.run_launcher.task_definition == task_definition["taskDefinitionArn"]

    with pytest.raises(
        Exception,
        match=r"You have attempted to fetch the environment variable FOO which is not set.",
    ):
        with instance_cm(
            config={"task_definition": {"env": "FOO"}, "container_name": "dagster"}
        ) as instance:
            print(instance.run_launcher)  # noqa: T201

    with environ({"FOO": "dagster"}):
        with instance_cm(
            config={"task_definition": {"env": "FOO"}, "container_name": "dagster"}
        ) as instance:
            assert instance.run_launcher.task_definition == task_definition["taskDefinitionArn"]



import pickle
from collections.abc import Callable
from typing import Any

import pytest
from dagster import (
    ConfigurableResource,
    DynamicOut,
    DynamicOutput,
    GraphIn,
    GraphOut,
    IAttachDifferentObjectToOpContext,
    In,
    Int,
    Out,
    Output,
    asset,
    graph,
    job,
    op,
    resource,
)
from dagster._core.definitions.assets.definition.assets_definition import AssetsDefinition
from dagster._core.definitions.definitions_class import Definitions
from dagster._core.definitions.partitions.definition import StaticPartitionsDefinition
from dagster._core.definitions.source_asset import SourceAsset
from dagster._core.definitions.unresolved_asset_job_definition import define_asset_job

from dagster_aws.s3.io_manager import S3PickleIOManager, s3_pickle_io_manager
from dagster_aws.s3.utils import construct_s3_client


class S3TestResource(ConfigurableResource, IAttachDifferentObjectToOpContext):
    def get_client(self) -> Any:
        return construct_s3_client(max_attempts=5)

    def get_object_to_set_on_execution_context(self) -> Any:
        return self.get_client()


@resource
def s3_test_resource(_):
    return construct_s3_client(max_attempts=5)


@pytest.fixture(name="s3_and_io_manager", params=[True, False])
def s3_and_io_manager_fixture(
    request,
) -> tuple[Any, Callable[[Any], Any]]:
    if request.param:
        return s3_test_resource, lambda _: s3_pickle_io_manager
    else:
        return (
            S3TestResource(),
            lambda s3: S3PickleIOManager.configure_at_launch(s3_resource=s3),
        )


def define_inty_job(s3_resource, s3_io_manager_builder):
    @op(out=Out(Int))
    def return_one():
        return 1

    @op(
        ins={"num": In(Int)},
        out=Out(Int),
    )
    def add_one(num):
        return num + 1

    @job(
        resource_defs={
            "io_manager": s3_io_manager_builder(s3_resource),
            "s3": s3_resource,
        }
    )
    def basic_external_plan_execution():
        add_one(return_one())

    return basic_external_plan_execution


def test_s3_pickle_io_manager_execution(mock_s3_bucket, s3_and_io_manager):
    assert not len(list(mock_s3_bucket.objects.all()))

    s3_resource, s3_io_manager_builder = s3_and_io_manager
    inty_job = define_inty_job(s3_resource, s3_io_manager_builder)

    run_config = {"resources": {"io_manager": {"config": {"s3_bucket": mock_s3_bucket.name}}}}

    result = inty_job.execute_in_process(run_config)

    assert result.output_for_node("return_one") == 1
    assert result.output_for_node("add_one") == 2

    assert len(list(mock_s3_bucket.objects.all())) == 2


def define_multiple_output_job():
    @op(
        out={
            "foo": Out(Int),
            "foobar": Out(Int),
        }
    )
    def return_two_outputs():
        yield Output(10, "foobar")
        yield Output(5, "foo")

    @job(resource_defs={"io_manager": s3_pickle_io_manager, "s3": s3_test_resource})
    def output_prefix_execution_plan():
        return_two_outputs()

    return output_prefix_execution_plan


def test_s3_pickle_io_manager_prefix(mock_s3_bucket):
    assert not len(list(mock_s3_bucket.objects.all()))

    prefixy_job = define_multiple_output_job()

    run_config = {"resources": {"io_manager": {"config": {"s3_bucket": mock_s3_bucket.name}}}}

    result = prefixy_job.execute_in_process(run_config)

    assert result.output_for_node("return_two_outputs", "foo") == 5
    assert result.output_for_node("return_two_outputs", "foobar") == 10

    assert len(list(mock_s3_bucket.objects.all())) == 2


def define_assets_job(bucket):
    @op
    def first_op(first_input):
        assert first_input == 4
        return first_input * 2

    @op
    def second_op(second_input):
        assert second_input == 8
        return second_input + 3

    source1 = SourceAsset("source1", partitions_def=StaticPartitionsDefinition(["foo", "bar"]))

    @asset
    def asset1(source1):
        return source1["foo"] + source1["bar"]

    @asset
    def asset2(asset1):
        assert asset1 == 3
        return asset1 + 1

    @graph(ins={"asset2": GraphIn()}, out={"asset3": GraphOut()})
    def graph_asset(asset2):
        return second_op(first_op(asset2))

    @asset(partitions_def=StaticPartitionsDefinition(["apple", "orange"]))
    def partitioned():
        return 8

    graph_asset_def = AssetsDefinition.from_graph(graph_asset)
    target_assets = [asset1, asset2, graph_asset_def, partitioned]

    return Definitions(
        assets=[*target_assets, source1],
        jobs=[define_asset_job("assets", target_assets)],
        resources={
            "io_manager": s3_pickle_io_manager.configured({"s3_bucket": bucket}),
            "s3": s3_test_resource,
        },
    ).resolve_job_def("assets")


def test_s3_pickle_io_manager_asset_execution(mock_s3_bucket):
    assert not len(list(mock_s3_bucket.objects.all()))
    inty_job = define_assets_job(mock_s3_bucket.name)
    # pickled_source1_foo = pickle.dumps(1)
    mock_s3_bucket.put_object(Key="dagster/source1/foo", Body=pickle.dumps(1))
    # pickled_source1_bar = pickle.dumps(2)
    mock_s3_bucket.put_object(Key="dagster/source1/bar", Body=pickle.dumps(2))

    result = inty_job.execute_in_process(partition_key="apple")

    assert result.output_for_node("asset1") == 3
    assert result.output_for_node("asset2") == 4
    assert result.output_for_node("graph_asset.first_op") == 8
    assert result.output_for_node("graph_asset.second_op") == 11

    objects = list(mock_s3_bucket.objects.all())
    assert len(objects) == 7
    assert {(o.bucket_name, o.key) for o in objects} == {
        ("test-bucket", "dagster/source1/bar"),
        ("test-bucket", "dagster/source1/foo"),
        ("test-bucket", "dagster/asset1"),
        ("test-bucket", "dagster/asset2"),
        ("test-bucket", "dagster/asset3"),
        ("test-bucket", "dagster/partitioned/apple"),
        (
            "test-bucket",
            "/".join(["dagster", "storage", result.run_id, "graph_asset.first_op", "result"]),
        ),
    }

    # re-execution does not cause issues, overwrites the buckets
    result2 = inty_job.execute_in_process(partition_key="apple")

    objects = list(mock_s3_bucket.objects.all())
    assert len(objects) == 8
    assert {(o.bucket_name, o.key) for o in objects} == {
        ("test-bucket", "dagster/source1/bar"),
        ("test-bucket", "dagster/source1/foo"),
        ("test-bucket", "dagster/asset1"),
        ("test-bucket", "dagster/asset2"),
        ("test-bucket", "dagster/asset3"),
        ("test-bucket", "dagster/partitioned/apple"),
        (
            "test-bucket",
            "/".join(["dagster", "storage", result.run_id, "graph_asset.first_op", "result"]),
        ),
        (
            "test-bucket",
            "/".join(["dagster", "storage", result2.run_id, "graph_asset.first_op", "result"]),
        ),
    }


def define_dynamic_job(s3_resource, s3_io_manager_builder):

    @op(out=DynamicOut())
    def dynamic_values():
        for key in ["foo", "bar"]:
            yield DynamicOutput(value=key, mapping_key=key)

    @op
    def return_value(value):
        return value

    @job(
        resource_defs={
            "io_manager": s3_io_manager_builder(s3_resource),
            "s3": s3_resource,
        }
    )
    def dynamic_job():
        dynamic_values().map(return_value)

    return dynamic_job


def test_s3_pickle_io_manager_dynamic_output(mock_s3_bucket, s3_and_io_manager):
    s3_resource, s3_io_manager_builder = s3_and_io_manager
    dynamic_job = define_dynamic_job(s3_resource, s3_io_manager_builder)

    run_config = {"resources": {"io_manager": {"config": {"s3_bucket": mock_s3_bucket.name}}}}

    result = dynamic_job.execute_in_process(run_config)

    assert result.success

    outputs = result.output_for_node("return_value")
    assert outputs == {"foo": "foo", "bar": "bar"}

    keys = [o.key for o in mock_s3_bucket.objects.all()]
    for key in keys:
        assert "[" not in key, f"S3 key contains '[': {key}"
        assert "]" not in key, f"S3 key contains ']': {key}"

    assert any("return_value--foo" in key for key in keys)
    assert any("return_value--bar" in key for key in keys)



from typing import get_args, get_origin

import pytest
from dagster._utils.test.definitions import scoped_definitions_load_context
from dagster.components.testing import create_defs_folder_sandbox

from dagster_aws.athena.resources import AthenaClientResource, ResourceWithAthenaConfig
from dagster_aws.components import (
    AthenaClientResourceComponent,
    AthenaCredentialsComponent,
    Boto3CredentialsComponent,
    ECRPublicResourceComponent,
    ParameterStoreResourceComponent,
    RDSResourceComponent,
    S3CredentialsComponent,
    S3FileManagerResourceComponent,
    S3ResourceComponent,
    SecretsManagerResourceComponent,
    SecretsManagerSecretsResourceComponent,
    SSMResourceComponent,
)
from dagster_aws.ecr import ECRPublicResource
from dagster_aws.rds.resources import RDSResource
from dagster_aws.redshift import RedshiftClientResourceComponent, RedshiftCredentialsComponent
from dagster_aws.redshift.resources import RedshiftClientResource
from dagster_aws.s3.resources import S3FileManagerResource, S3Resource
from dagster_aws.secretsmanager.resources import (
    SecretsManagerResource,
    SecretsManagerSecretsResource,
)
from dagster_aws.ssm.resources import ParameterStoreResource, SSMResource
from dagster_aws.utils import ResourceWithBoto3Configuration

# --- Field Sync Tests ---


@pytest.mark.parametrize(
    "component_class, resource_class",
    [
        (Boto3CredentialsComponent, ResourceWithBoto3Configuration),
        (S3CredentialsComponent, S3Resource),
        (AthenaCredentialsComponent, ResourceWithAthenaConfig),
        (RedshiftCredentialsComponent, RedshiftClientResource),
        (S3ResourceComponent, S3Resource),
        (S3FileManagerResourceComponent, S3FileManagerResource),
        (AthenaClientResourceComponent, AthenaClientResource),
        (RedshiftClientResourceComponent, RedshiftClientResource),
        (SSMResourceComponent, SSMResource),
        (ParameterStoreResourceComponent, ParameterStoreResource),
        (SecretsManagerResourceComponent, SecretsManagerResource),
        (SecretsManagerSecretsResourceComponent, SecretsManagerSecretsResource),
        (RDSResourceComponent, RDSResource),
        (ECRPublicResourceComponent, ECRPublicResource),
    ],
)
def test_component_fields_sync_with_resource(component_class, resource_class):
    """Validates that component configuration schemas are fully synchronized with their underlying resource definitions."""
    component_fields = set(component_class.model_fields.keys())

    if "credentials" in component_class.model_fields:
        creds_field = component_class.model_fields["credentials"]
        creds_annotation = creds_field.annotation
        origin = get_origin(creds_annotation)
        if origin:
            args = get_args(creds_annotation)
            actual_creds_class = next((arg for arg in args if hasattr(arg, "model_fields")), None)
        else:
            actual_creds_class = creds_annotation

        if actual_creds_class and hasattr(actual_creds_class, "model_fields"):
            component_fields.update(actual_creds_class.model_fields.keys())

    resource_fields = set(resource_class.model_fields.keys())
    missing = resource_fields - component_fields
    assert resource_fields.issubset(component_fields), (
        f"Missing fields in {component_class.__name__}: {missing}"
    )


# --- S3 Component Tests ---


def test_s3_component_integration(monkeypatch):
    """Verifies successful instantiation of S3Resource from YAML with environment variable resolution."""
    monkeypatch.setenv("AWS_REGION", "us-east-1")
    monkeypatch.setenv("MAX_ATTEMPTS", "5")

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=S3ResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.S3ResourceComponent",
                "attributes": {
                    "credentials": {
                        "region_name": "{{ env.AWS_REGION }}",
                        "max_attempts": "{{ env.MAX_ATTEMPTS }}",
                    },
                    "resource_key": "my_s3",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert isinstance(defs.resources["my_s3"], S3Resource)
                assert defs.resources["my_s3"].region_name == "us-east-1"
                assert defs.resources["my_s3"].max_attempts == 5


def test_s3_no_resource_key():
    """Verifies that when resource_key is not provided, the resources dict is empty."""
    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=S3ResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.S3ResourceComponent",
                "attributes": {
                    "credentials": {"region_name": "us-east-1"},
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources == {}


def test_s3_explicit_key():
    """Verifies that the component registers correctly when resource_key is provided."""
    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=S3ResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.S3ResourceComponent",
                "attributes": {
                    "credentials": {"region_name": "us-east-1"},
                    "resource_key": "s3",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert "s3" in defs.resources
                assert isinstance(defs.resources["s3"], S3Resource)


def test_s3_file_manager_integration(monkeypatch):
    """Validates S3FileManagerResourceComponent instantiation and parameter mapping."""
    monkeypatch.setenv("AWS_REGION", "us-east-1")
    monkeypatch.setenv("MY_BUCKET", "test-bucket-artifacts")
    monkeypatch.setenv("MY_PREFIX", "dagster/logs")

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=S3FileManagerResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.S3FileManagerResourceComponent",
                "attributes": {
                    "credentials": {"region_name": "{{ env.AWS_REGION }}"},
                    "s3_bucket": "{{ env.MY_BUCKET }}",
                    "s3_prefix": "{{ env.MY_PREFIX }}",
                    "resource_key": "my_file_manager",
                },
            },
        )

        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert "my_file_manager" in defs.resources
                resource = defs.resources["my_file_manager"]
                assert isinstance(resource, S3FileManagerResource)
                assert resource.s3_bucket == "test-bucket-artifacts"
                assert resource.s3_prefix == "dagster/logs"
                assert resource.region_name == "us-east-1"


# --- Athena Component Tests ---


def test_athena_component_integration(monkeypatch):
    """Verifies AthenaClientResource loading with nested credentials configuration."""
    monkeypatch.setenv("ATHENA_WORKGROUP", "primary")
    monkeypatch.setenv("AWS_REGION", "us-west-2")

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=AthenaClientResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.AthenaClientResourceComponent",
                "attributes": {
                    "credentials": {
                        "region_name": "{{ env.AWS_REGION }}",
                        "workgroup": "{{ env.ATHENA_WORKGROUP }}",
                    },
                    "resource_key": "athena",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert defs.resources["athena"].workgroup == "primary"


# --- Redshift Component Tests ---


def test_redshift_component_integration(monkeypatch):
    """Verifies RedshiftClientResource loading and type conversion (string env vars to integer ports)."""
    monkeypatch.setenv("REDSHIFT_HOST", "cluster.redshift.amazonaws.com")

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=RedshiftClientResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.redshift.RedshiftClientResourceComponent",
                "attributes": {
                    "credentials": {
                        "host": "{{ env.REDSHIFT_HOST }}",
                        "port": 5439,
                        "database": "dev",
                        "user": "admin",
                    },
                    "resource_key": "redshift",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert defs.resources["redshift"].host == "cluster.redshift.amazonaws.com"
                assert defs.resources["redshift"].port == 5439


# --- SSM Component Tests ---


def test_ssm_component_integration(monkeypatch):
    """Verifies SSMResource loading with basic regional configuration."""
    monkeypatch.setenv("AWS_REGION", "us-east-1")

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=SSMResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.SSMResourceComponent",
                "attributes": {
                    "credentials": {"region_name": "{{ env.AWS_REGION }}"},
                    "resource_key": "ssm",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert defs.resources["ssm"].region_name == "us-east-1"


# --- Parameter Store Component Tests ---


def test_parameter_store_defaults_behavior():
    """Verifies that ParameterStoreResource defaults to an empty parameter list when not specified."""
    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=ParameterStoreResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.ParameterStoreResourceComponent",
                "attributes": {
                    "credentials": {"region_name": "us-east-1"},
                    "resource_key": "parameter_store",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert defs.resources["parameter_store"].parameters == []


def test_parameter_store_complex_integration(monkeypatch):
    """Verifies ParameterStoreResource handling of complex types (lists) and boolean flags."""
    monkeypatch.setenv("AWS_REGION", "us-west-1")

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=ParameterStoreResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.ParameterStoreResourceComponent",
                "attributes": {
                    "credentials": {"region_name": "{{ env.AWS_REGION }}"},
                    "parameters": ["/app/db/password"],
                    "with_decryption": True,
                    "resource_key": "parameter_store",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                res = defs.resources["parameter_store"]
                assert res.region_name == "us-west-1"
                assert res.parameters == ["/app/db/password"]
                assert res.with_decryption is True


# --- Secrets Manager Component Tests ---


def test_secrets_manager_integration(monkeypatch):
    """Verifies SecretsManagerResource (Client) instantiation from configuration."""
    monkeypatch.setenv("AWS_REGION", "us-east-1")

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=SecretsManagerResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.SecretsManagerResourceComponent",
                "attributes": {
                    "credentials": {"region_name": "{{ env.AWS_REGION }}"},
                    "resource_key": "secretsmanager",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert isinstance(defs.resources["secretsmanager"], SecretsManagerResource)
                assert defs.resources["secretsmanager"].region_name == "us-east-1"


def test_secrets_manager_secrets_integration(monkeypatch):
    """Verifies SecretsManagerSecretsResource loads specific secret ARNs correctly from YAML."""
    monkeypatch.setenv("SECRET_ARN", "arn:aws:secretsmanager:us-east-1:123:secret:db")

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=SecretsManagerSecretsResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.SecretsManagerSecretsResourceComponent",
                "attributes": {
                    "credentials": {"region_name": "us-east-1"},
                    "secrets": ["{{ env.SECRET_ARN }}"],
                    "resource_key": "my_secrets",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert defs.resources["my_secrets"].secrets == [
                    "arn:aws:secretsmanager:us-east-1:123:secret:db"
                ]


# --- RDS Component Tests ---


def test_rds_component_integration(monkeypatch):
    """Verifies RDSResource loading with endpoint URL and region configuration."""
    monkeypatch.setenv("RDS_ENDPOINT", "https://rds.amazonaws.com")
    monkeypatch.setenv("AWS_REGION", "eu-central-1")

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=RDSResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.RDSResourceComponent",
                "attributes": {
                    "credentials": {
                        "endpoint_url": "{{ env.RDS_ENDPOINT }}",
                        "region_name": "{{ env.AWS_REGION }}",
                    },
                    "resource_key": "production_db",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert isinstance(defs.resources["production_db"], RDSResource)
                assert defs.resources["production_db"].endpoint_url == "https://rds.amazonaws.com"


# --- ECR Component Tests ---


def test_ecr_public_component_integration(monkeypatch):
    """Verifies ECRPublicResource instantiation via the Dagster Sandbox."""
    monkeypatch.setenv("AWS_REGION", "us-east-1")

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=ECRPublicResourceComponent,
            defs_yaml_contents={
                "type": "dagster_aws.components.ECRPublicResourceComponent",
                "attributes": {
                    "credentials": {"region_name": "{{ env.AWS_REGION }}"},
                    "resource_key": "public_ecr",
                },
            },
        )
        with scoped_definitions_load_context():
            with sandbox.load_component_and_build_defs(defs_path=defs_path) as (_component, defs):
                assert defs.resources
                assert "public_ecr" in defs.resources
                resource = defs.resources["public_ecr"]
                assert isinstance(resource, ECRPublicResource)



import fsspec
import pytest
import s3fs
from dagster._core.instance_for_test import instance_for_test
from dagster._core.storage.defs_state.blob_storage_state_storage import UPathDefsStateStorage
from dagster._core.test_utils import ensure_dagster_tests_import
from dagster_shared import check
from moto.server import ThreadedMotoServer  # type: ignore  # (no type stubs)

ensure_dagster_tests_import()
from dagster_tests.storage_tests.utils.defs_state_storage import TestDefsStateStorage

_MOCK_STORAGE_OPTIONS = {
    "anon": False,
    "key": "fake",
    "secret": "fake",
    "client_kwargs": {"endpoint_url": "http://localhost:5001"},
}


@pytest.fixture(scope="class", autouse=True)
def moto_s3_autopatch():
    """Because s3fs doesn't play nicely with moto, we need some additional mocking to get it to work."""
    server = ThreadedMotoServer(port=5001)
    server.start()

    fsspec.register_implementation("s3", s3fs.S3FileSystem, clobber=True)

    yield

    server.stop()


class TestS3UPathDefsStateStorage(TestDefsStateStorage):
    """Tests the blob storage state storage implementation."""

    __test__ = True

    @pytest.fixture(name="storage", scope="function")
    def state_storage(self, mock_s3_bucket):
        with instance_for_test(
            overrides={
                "defs_state_storage": {
                    "module": "dagster._core.storage.defs_state.blob_storage_state_storage",
                    "class": "UPathDefsStateStorage",
                    "config": {
                        "base_path": f"s3://{mock_s3_bucket.name}/foo",
                        "storage_options": _MOCK_STORAGE_OPTIONS,
                    },
                }
            }
        ) as instance:
            state_storage = check.inst(instance.defs_state_storage, UPathDefsStateStorage)

            # Check that we have an S3 filesystem
            assert state_storage.base_path.fs.__class__.__name__ == "S3FileSystem"
            yield state_storage



from typing import TYPE_CHECKING
from uuid import uuid4

import boto3
import pytest
from dagster import asset, materialize, open_pipes_session
from dagster._core.execution.context.compute import AssetExecutionContext
from dagster._core.instance_for_test import instance_for_test
from dagster._core.pipes.utils import PipesEnvContextInjector

from dagster_aws.pipes import PipesCloudWatchMessageReader, PipesEMRServerlessClient
from dagster_aws_tests.pipes_tests.utils import _MOTO_SERVER_URL

if TYPE_CHECKING:
    from mypy_boto3_emr_serverless import EMRServerlessClient


EMR_SERVERLESS_APP_NAME = "Example"


@pytest.fixture
def emr_serverless_setup(moto_server, s3_client) -> tuple["EMRServerlessClient", str]:
    client = boto3.client("emr-serverless", region_name="us-east-1", endpoint_url=_MOTO_SERVER_URL)
    resp = client.create_application(
        type="SPARK",
        releaseLabel="emr-7.2.0-latest",
        clientToken=str(uuid4()),
    )
    return client, resp["applicationId"]


def test_emr_serverless_manual(emr_serverless_setup: tuple["EMRServerlessClient", str]):
    client, application_id = emr_serverless_setup

    @asset
    def my_asset(context: AssetExecutionContext, emr_serverless_client: PipesEMRServerlessClient):
        message_reader = PipesCloudWatchMessageReader()
        context_injector = PipesEnvContextInjector()

        with open_pipes_session(
            context=context,
            message_reader=message_reader,
            context_injector=context_injector,
        ) as session:
            params = emr_serverless_client._enrich_start_params(  # noqa: SLF001
                context=context,
                session=session,
                params={
                    "applicationId": application_id,
                    "executionRoleArn": "arn:aws:iam::123456789012:role/EMRServerlessRole",
                    "jobDriver": {
                        "sparkSubmit": {
                            "entryPoint": "s3://my-bucket/my-script.py",
                        }
                    },
                    "clientToken": str(uuid4()),
                },
            )

            assert params["tags"]["dagster/run-id"] == context.run_id  # pyright: ignore[reportTypedDictNotRequiredAccess]
            assert (
                "--conf spark.emr-serverless.driverEnv.DAGSTER_PIPES_CONTEXT="
                in params["jobDriver"]["sparkSubmit"]["sparkSubmitParameters"]  # pyright: ignore[reportTypedDictNotRequiredAccess]
            )
            assert (
                "--conf spark.emr-serverless.driverEnv.DAGSTER_PIPES_MESSAGES="
                in params["jobDriver"]["sparkSubmit"]["sparkSubmitParameters"]  # pyright: ignore[reportTypedDictNotRequiredAccess]
            )

            # moto doesn't have start_job_run implemented so this is as far as we can get with it right now

            return session.get_results()

    with instance_for_test() as instance:
        materialize(
            [my_asset],
            resources={
                "emr_serverless_client": PipesEMRServerlessClient(
                    client=client,
                )
            },
            instance=instance,
        )



import requests
from dagster import asset, materialize_to_memory

from dagster_aws.rds import RDSResource


def configure_moto_mock_response():
    mock_response = {
        "results": [
            {
                "records": [
                    [{"stringValue": "title1"}, {"longValue": 2001}],
                    [{"stringValue": "title2"}, {"longValue": 2010}],
                ],
                "numberOfRecordsUpdated": 0,
            }
        ]
    }

    # moto is configured through HTTP: https://docs.getmoto.org/en/stable/docs/services/rds-data.html
    resp = requests.post(
        "http://motoapi.amazonaws.com/moto-api/static/rds-data/statement-results",
        json=mock_response,
    )
    assert resp.status_code == 201


def test_rds_resource(mock_rds_client, mock_rds_data_client):
    @asset
    def rds_asset(rds_resource: RDSResource):
        with rds_resource.get_rds_client() as rds_client:
            rds_client.create_db_instance(
                DBInstanceIdentifier="test-instance",
                MasterUsername="admin",
                MasterUserPassword="password",
                DBInstanceClass="db.t2.micro",
                Engine="postgres",
            )
            db_instances = rds_client.describe_db_instances()["DBInstances"]
            assert len(db_instances) == 1

    @asset
    def rds_data_asset(rds_resource: RDSResource):
        configure_moto_mock_response()

        with rds_resource.get_data_client() as rds_data_client:
            response = rds_data_client.execute_statement(
                resourceArn="arn:aws:rds:us-east-1:123456789012:cluster:database-1",
                secretArn="arn:aws:secretsmanager:us-east-1:123456789012:secret:rds!cluster-1",
                sql="SELECT * from films;",
            )

            assert len(response["records"]) == 2

    result = materialize_to_memory(
        [rds_asset, rds_data_asset],
        resources={"rds_resource": RDSResource(region_name="us-east-1")},
    )

    assert result.success



import sys
import time
import uuid
from dataclasses import dataclass
from datetime import datetime
from subprocess import PIPE, Popen
from typing import cast

import boto3
import botocore


@dataclass
class SimulatedTaskRun:
    popen: Popen | None
    cluster: str
    task_arn: str
    log_group: str
    log_stream: str
    created_at: datetime
    runtime_id: str
    stopped_reason: str | None = None
    stop_code: str | None = None
    stopped: bool = False
    logs_uploaded: bool = False


class LocalECSMockClient:
    CONTAINER_NAME = "test-container"

    def __init__(self, ecs_client: boto3.client, cloudwatch_client: boto3.client):  # pyright: ignore (reportGeneralTypeIssues)
        self.ecs_client = ecs_client
        self.cloudwatch_client = cloudwatch_client

        self._task_runs: dict[
            str, SimulatedTaskRun
        ] = {}  # mapping of TaskDefinitionArn to TaskDefinition

    @property
    def meta(self):
        return self.ecs_client.meta

    def get_waiter(self, waiter_name: str):
        return WaiterMock(self, waiter_name)

    def register_task_definition(self, **kwargs):
        return self.ecs_client.register_task_definition(**kwargs)

    def describe_task_definition(self, **kwargs):
        response = self.ecs_client.describe_task_definition(**kwargs)
        assert len(response["taskDefinition"]["containerDefinitions"]) == 1, (
            "Only 1 container is supported in tests"
        )
        # unlike real ECS, moto doesn't use cloudwatch logging by default
        # so let's add it here
        response["taskDefinition"]["containerDefinitions"][0]["logConfiguration"] = (
            response["taskDefinition"]["containerDefinitions"][0].get("logConfiguration")
            or {
                "logDriver": "awslogs",
                "options": {
                    "awslogs-group": f"{response['taskDefinition']['taskDefinitionArn']}",  # this value doesn't really matter
                    "awslogs-stream-prefix": "ecs",
                },
            }
        )
        return response

    def run_task(self, **kwargs):
        response = self.ecs_client.run_task(**kwargs)

        # inject container name in case it's missing
        response["tasks"][0]["containers"] = response["tasks"][0].get("containers") or []

        if len(response["tasks"][0]["containers"]) == 0:
            response["tasks"][0]["containers"].append({})

        response["tasks"][0]["containers"][0]["name"] = (
            response["tasks"][0]["containers"][0].get("name") or self.CONTAINER_NAME
        )

        task_arn = response["tasks"][0]["taskArn"]
        task_definition_arn = response["tasks"][0]["taskDefinitionArn"]

        task_definition = self.describe_task_definition(taskDefinition=task_definition_arn)[
            "taskDefinition"
        ]

        assert len(task_definition["containerDefinitions"]) == 1, (
            "Only 1 container is supported in tests"
        )

        # execute in a separate process
        command = task_definition["containerDefinitions"][0]["command"]

        assert command[0] == sys.executable, (
            "Only the current Python interpreter is supported in tests"
        )

        created_at = datetime.now()

        popen = Popen(
            command,
            stdout=PIPE,
            stderr=PIPE,
            # get env from container overrides
            env={
                env["name"]: env["value"]
                for env in kwargs["overrides"]["containerOverrides"][0].get("environment", [])
            },
        )

        log_group = task_definition["containerDefinitions"][0]["logConfiguration"]["options"][
            "awslogs-group"
        ]
        stream_prefix = task_definition["containerDefinitions"][0]["logConfiguration"]["options"][
            "awslogs-stream-prefix"
        ]
        container_name = task_definition["containerDefinitions"][0]["name"]
        log_stream = f"{stream_prefix}/{container_name}/{task_arn.split('/')[-1]}"

        self._task_runs[task_arn] = SimulatedTaskRun(
            popen=popen,
            cluster=kwargs.get("cluster", "default"),
            task_arn=task_arn,
            log_group=log_group,
            log_stream=log_stream,
            created_at=created_at,
            runtime_id=str(uuid.uuid4()),
        )

        self._create_cloudwatch_streams(task_arn)

        return response

    def describe_tasks(self, cluster: str, tasks: list[str]):
        assert len(tasks) == 1, "Only 1 task is supported in tests"

        simulated_task = cast("SimulatedTaskRun", self._task_runs[tasks[0]])

        response = self.ecs_client.describe_tasks(cluster=cluster, tasks=tasks)

        assert len(response["tasks"]) == 1, "Only 1 task is supported in tests"

        task_definition = self.describe_task_definition(
            taskDefinition=response["tasks"][0]["taskDefinitionArn"]
        )["taskDefinition"]

        assert len(task_definition["containerDefinitions"]) == 1, (
            "Only 1 container is supported in tests"
        )

        # need to inject container name since moto doesn't return it

        response["tasks"][0]["containers"].append(
            {
                "name": task_definition["containerDefinitions"][0]["name"],
                "runtimeId": simulated_task.runtime_id,
            }
        )

        response["tasks"][0]["createdAt"] = simulated_task.created_at

        # check if any failed
        for task in response["tasks"]:
            if task["taskArn"] in self._task_runs:
                simulated_task = self._task_runs[task["taskArn"]]

                if simulated_task.stop_code == "TaskFailedToStart":
                    task["lastStatus"] = "STOPPED"
                    task["stopCode"] = "TaskFailedToStart"
                    task["stoppedReason"] = simulated_task.stopped_reason
                    return response

                if simulated_task.stopped:
                    task["lastStatus"] = "STOPPED"
                    task["stoppedReason"] = simulated_task.stopped_reason
                    task["containers"][0]["exitCode"] = 1
                    self._upload_logs_to_cloudwatch(task["taskArn"])
                    return response

                assert simulated_task.popen is not None
                if simulated_task.popen.poll() is not None:
                    simulated_task.popen.wait()
                    # check status code
                    if simulated_task.popen.returncode == 0:
                        task["lastStatus"] = "STOPPED"
                        task["containers"][0]["exitCode"] = 0
                    else:
                        task["lastStatus"] = "STOPPED"
                        # _, stderr = simulated_task.popen.communicate()
                        task["containers"][0]["exitCode"] = 1

                    self._upload_logs_to_cloudwatch(task["taskArn"])

                else:
                    task["lastStatus"] = "RUNNING"

        return response

    def simulate_task_failed_to_start(self, task_arn: str, reason: str):
        """Simulate a task that failed to start (e.g. image pull failure, missing IAM permissions)."""
        if simulated_task := self._task_runs.get(task_arn):
            simulated_task.stop_code = "TaskFailedToStart"
            simulated_task.stopped_reason = reason
            simulated_task.stopped = True
            if simulated_task.popen is not None:
                simulated_task.popen.terminate()

    def stop_task(self, cluster: str, task: str, reason: str | None = None):
        if simulated_task := self._task_runs.get(task):
            assert simulated_task.popen is not None
            simulated_task.popen.terminate()
            simulated_task.stopped = True
            simulated_task.stopped_reason = reason
            self._upload_logs_to_cloudwatch(task)
        else:
            raise RuntimeError(f"Task {task} was not found")

    def _create_cloudwatch_streams(self, task: str):
        simulated_task = self._task_runs[task]

        log_group = simulated_task.log_group
        log_stream = simulated_task.log_stream

        try:
            self.cloudwatch_client.create_log_group(
                logGroupName=f"{log_group}",
            )
        except self.cloudwatch_client.exceptions.ResourceAlreadyExistsException:
            pass

        try:
            self.cloudwatch_client.create_log_stream(
                logGroupName=f"{log_group}",
                logStreamName=log_stream,
            )
        except self.cloudwatch_client.exceptions.ResourceAlreadyExistsException:
            pass

    def _upload_logs_to_cloudwatch(self, task: str):
        simulated_task = self._task_runs[task]

        if simulated_task.logs_uploaded:
            return

        if simulated_task.popen is None:
            simulated_task.logs_uploaded = True
            return

        log_group = simulated_task.log_group
        log_stream = simulated_task.log_stream

        stdout, stderr = simulated_task.popen.communicate()

        for out in [stderr, stdout]:
            for line in out.decode().split("\n"):
                if line:
                    self.cloudwatch_client.put_log_events(
                        logGroupName=f"{log_group}",
                        logStreamName=log_stream,
                        logEvents=[{"timestamp": int(time.time() * 1000), "message": str(line)}],
                    )

        time.sleep(0.1)

        simulated_task.logs_uploaded = True


class WaiterMock:
    def __init__(self, client: LocalECSMockClient, waiter_name: str):
        self.client = client
        self.waiter_name = waiter_name

    def wait(self, **kwargs):
        waiter_config = kwargs.pop("WaiterConfig", {"MaxAttempts": 100, "Delay": 6})
        max_attempts = int(waiter_config.get("MaxAttempts", 100))
        delay = int(waiter_config.get("Delay", 6))
        num_attempts = 0

        if self.waiter_name == "tasks_stopped":
            while True:
                response = self.client.describe_tasks(**kwargs)
                num_attempts += 1

                if all(task["lastStatus"] == "STOPPED" for task in response["tasks"]):
                    return

                if num_attempts >= max_attempts:
                    raise botocore.exceptions.WaiterError(  # pyright: ignore[reportAttributeAccessIssue]
                        name=self.waiter_name,
                        reason="Max attempts exceeded",
                        last_response=response,
                    )

                time.sleep(delay)

        else:
            raise NotImplementedError(f"Waiter {self.waiter_name} is not implemented")



from typing import Any

import pytest
from dagster import DagsterEventType, job, op
from dagster._core.definitions.metadata import MetadataValue

from dagster_aws.s3 import (
    S3FileHandle,
    S3FileManagerResource,
    S3Resource,
    file_handle_to_s3,
    s3_file_manager,
    s3_resource,
)


@pytest.fixture(name="s3_resource_type", params=[True, False])
def s3_resource_type_fixture(request) -> Any:
    if request.param:
        return s3_resource
    else:
        return S3Resource.configure_at_launch()


@pytest.fixture(name="s3_file_manager_resource_type", params=[True, False])
def s3_file_manager_resource_type_fixture(request) -> Any:
    if request.param:
        return s3_file_manager
    else:
        return S3FileManagerResource.configure_at_launch()


def create_file_handle_job(temp_file_handle, s3_resource_type, s3_file_manager_resource_type):
    @op
    def emit_temp_handle(_):
        return temp_file_handle

    @job(resource_defs={"s3": s3_resource_type, "file_manager": s3_file_manager_resource_type})
    def test():
        file_handle_to_s3(emit_temp_handle())

    return test


def test_successful_file_handle_to_s3(
    mock_s3_bucket, s3_resource_type, s3_file_manager_resource_type
):
    foo_bytes = b"foo"
    remote_s3_object = mock_s3_bucket.Object("some-key/foo")
    remote_s3_object.put(Body=foo_bytes)

    file_handle = S3FileHandle(mock_s3_bucket.name, "some-key/foo")
    result = create_file_handle_job(
        file_handle, s3_resource_type, s3_file_manager_resource_type
    ).execute_in_process(
        run_config={
            "ops": {
                "file_handle_to_s3": {"config": {"Bucket": mock_s3_bucket.name, "Key": "some-key"}}
            },
            "resources": {"file_manager": {"config": {"s3_bucket": mock_s3_bucket.name}}},
        },
    )

    assert result.success

    assert mock_s3_bucket.Object(key="some-key").get()["Body"].read() == foo_bytes

    materializations = [
        event.step_materialization_data.materialization
        for event in result.events_for_node("file_handle_to_s3")
        if event.event_type == DagsterEventType.ASSET_MATERIALIZATION
    ]
    assert len(materializations) == 1
    assert len(materializations[0].metadata) == 1
    assert materializations[0].metadata["some-key"] == MetadataValue.path(
        f"s3://{mock_s3_bucket.name}/some-key"
    )



# ruff: isort: skip_file
# fmt: off
# start_marker
from dagster_aws.ecs import ecs_executor

from dagster import job, op


@op(
    tags={"ecs/cpu": "256", "ecs/memory": "512"},
)
def ecs_op():
    pass


@job(executor_def=ecs_executor)
def ecs_job():
    ecs_op()


# end_marker
# fmt: on


def test_mode():
    assert ecs_job



import boto3
import pytest
from moto import mock_aws

from dagster_aws.athena.resources import (
    AthenaError,
    AthenaTimeout,
    FakeAthenaClient,
    ResourceWithAthenaConfig,
    fake_athena_resource,
)


class TestAthenaClientResource(ResourceWithAthenaConfig):
    def get_client(self) -> FakeAthenaClient:
        return FakeAthenaClient(
            client=boto3.client("athena", region_name="us-east-1"),
            workgroup=self.workgroup,
            polling_interval=self.polling_interval,
            max_polls=self.max_polls,
        )


@pytest.fixture
def mock_athena_client(mock_s3_resource):
    with mock_aws():
        yield boto3.client("athena", region_name="us-east-1")


def test_execute_query(mock_athena_client):
    athena = FakeAthenaClient(client=mock_athena_client)
    assert athena.execute_query("SELECT 1", fetch_results=True) == [("1",)]
    assert athena.execute_query(
        "SELECT * FROM foo", fetch_results=True, expected_results=[(1, None), (2, 3)]
    ) == [("1",), ("2", "3")]


@pytest.mark.parametrize(
    "expected_states",
    [
        ["SUCCEEDED"],
        ["QUEUED", "SUCCEEDED"],
        ["QUEUED", "RUNNING", "SUCCEEDED"],
        ["QUEUED", "QUEUED", "SUCCEEDED"],
        ["QUEUED", "RUNNING", "RUNNING", "SUCCEEDED"],
    ],
)
def test_execute_query_state_transitions(mock_athena_client, expected_states):
    athena = FakeAthenaClient(client=mock_athena_client)
    athena.execute_query("SELECT 1", expected_states=expected_states)


@pytest.mark.parametrize(
    "expected_states",
    [
        ["FAILED"],
        ["CANCELLED"],
        ["QUEUED", "FAILED"],
        ["QUEUED", "RUNNING", "FAILED"],
        ["QUEUED", "CANCELLED"],
        ["QUEUED", "RUNNING", "CANCELLED"],
    ],
)
def test_execute_query_raises(mock_athena_client, expected_states):
    athena = FakeAthenaClient(client=mock_athena_client)
    with pytest.raises(AthenaError, match="state change reason"):
        athena.execute_query("SELECT 1", expected_states=expected_states)


def test_execute_query_timeout(mock_athena_client):
    athena = FakeAthenaClient(client=mock_athena_client, max_polls=1)
    with pytest.raises(AthenaTimeout):
        athena.execute_query("SELECT 1")


def test_execute_query_succeeds_on_last_poll(mock_athena_client):
    athena = FakeAthenaClient(client=mock_athena_client, max_polls=1)
    athena.execute_query("SELECT 1", expected_states=["SUCCEEDED"])


def test_op(mock_athena_client) -> None:
    from dagster import build_op_context, op

    @op(required_resource_keys={"athena"})
    def example_athena_op(context):
        return context.resources.athena.execute_query("SELECT 1", fetch_results=True)

    context = build_op_context(resources={"athena": fake_athena_resource})
    assert example_athena_op(context) == [("1",)]


def test_op_pythonic_resource(mock_athena_client) -> None:
    from dagster import op

    @op
    def example_athena_op(athena: TestAthenaClientResource):
        return athena.get_client().execute_query("SELECT 1", fetch_results=True)

    assert example_athena_op(athena=TestAthenaClientResource.configure_at_launch()) == [("1",)]
